import { Component, Input, OnInit } from '@angular/core';
import { GpTemplatesCode, GpTitolare, OwnerProcedure, ProcedureTemplateConfig, VariableGp } from 'src/app/models';
import { GpValueVariablePipe } from 'src/app/pipe';
import { TEMPLATES_CODE, TEMPLATE_PROCEDURE } from 'src/app/utility';

@Component({
  selector: 'app-gp-procedure-owner',
  templateUrl: './gp-procedure-owner.component.html',
  styleUrls: ['./gp-procedure-owner.component.scss'],
})
export class GpProcedureOwnerComponent implements OnInit {
  @Input() variablesGp: VariableGp[];
  @Input() templateConfiguration: ProcedureTemplateConfig;
  templatesCodes: GpTemplatesCode = TEMPLATES_CODE;
  templateProcedure = TEMPLATE_PROCEDURE;
  arrow_down: string = 'assets/icon/icona_sotto.svg';
  arrow_right: string = 'assets/icon/icona_avanti.svg';
  showCardContent: boolean;
  iconArrow: string;
  ownerProcedure: OwnerProcedure;

  constructor(
    public gpValueVariablePipe: GpValueVariablePipe,
  ) {
    this.setUtility();
  }

  ngOnInit() {
  }

  setUtility() {
    this.iconArrow = this.arrow_right;
    setTimeout(() => {
      this.openCard();
    }, 300)
  }

  openCard() {
    this.showCardContent = true;
    this.iconArrow = this.arrow_down;
  }

  manageCloseOpen() {
    if (this.showCardContent) {
      this.showCardContent = false;
      this.iconArrow = this.arrow_right;
    } else {
      this.showCardContent = true;
      this.iconArrow = this.arrow_down;
    }
  }

  checkAndReturnOwner(): OwnerProcedure {
    let propImmobile = this.gpValueVariablePipe.transform(this.templateProcedure?.codeVariables.propImmobile, this.variablesGp);

    if (propImmobile && propImmobile === 's') {
      if (this.gpValueVariablePipe.transform(this.templateProcedure?.codeVariables.codiceFiscale, this.variablesGp) ||
        this.gpValueVariablePipe.transform(this.templateProcedure?.codeVariables.nome, this.variablesGp) ||
        this.gpValueVariablePipe.transform(this.templateProcedure?.codeVariables.cognome, this.variablesGp)) {
        this.ownerProcedure = {
          codiceFiscale: this.gpValueVariablePipe.transform(this.templateProcedure?.codeVariables.codiceFiscale, this.variablesGp) as string,
          nome: this.gpValueVariablePipe.transform(this.templateProcedure?.codeVariables.nome, this.variablesGp) as string,
          cognome: this.gpValueVariablePipe.transform(this.templateProcedure?.codeVariables.cognome, this.variablesGp) as string
        };
      } else {
        return null;
      }

    } else if (propImmobile && propImmobile === 'n') {
      if (
        this.gpValueVariablePipe.transform(this.templateProcedure?.codeVariables.codiceFiscaleProprietario, this.variablesGp) ||
        this.gpValueVariablePipe.transform(this.templateProcedure?.codeVariables.nomeProprietario, this.variablesGp) ||
        this.gpValueVariablePipe.transform(this.templateProcedure?.codeVariables.cognomeProprietario, this.variablesGp)) {
          this.ownerProcedure = {
          codiceFiscale: this.gpValueVariablePipe.transform(this.templateProcedure?.codeVariables.codiceFiscaleProprietario, this.variablesGp) as string,
          nome: this.gpValueVariablePipe.transform(this.templateProcedure?.codeVariables.nomeProprietario, this.variablesGp) as string,
          cognome: this.gpValueVariablePipe.transform(this.templateProcedure?.codeVariables.cognomeProprietario, this.variablesGp) as string
        };
      } else {
        return null;
      }
    } else {
      return null;
    }
    return this.ownerProcedure;
  }

  getTitolariFromVars(): GpTitolare[] {
    try {
      let titolari: GpTitolare[] = JSON.parse(this.gpValueVariablePipe.transform(this.templateProcedure.codeVariables.titolari, this.variablesGp) as string);
      return titolari && titolari.length > 0 ? titolari : [];
    } catch (e) {
      return [];
    }
  }

}