import { Injectable } from '@angular/core';
import { CanActivate, ActivatedRouteSnapshot, RouterStateSnapshot, UrlTree, UrlSegment, Route } from '@angular/router';
import { NavController } from '@ionic/angular';
import { AuthService } from 'ionic-appauth';
import { Observable } from 'rxjs';
import { filter, switchMap, tap } from 'rxjs/operators';
import { AuthenticationService } from 'src/app/services/authentication/authentication.service';
import { PAGES } from 'src/app/utility';

@Injectable({
  providedIn: 'root'
})
export class AuthGuard implements CanActivate {

  constructor(
    private auth: AuthService,
    private authenticationService: AuthenticationService,
    private navCtrl: NavController/*  */) { }

  canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): boolean | Observable<boolean> | Promise<boolean> {
    return this.authenticationService.isLoggedIn().then((isLoggedIn) => {
      if (!isLoggedIn) {
        this.navCtrl.navigateRoot(PAGES.landing_page)
        return false;
      }
      return true;
    });
    /*    
    ATTIVARE IN PROD
    return this.auth.initComplete$.pipe(
          filter(complete => complete),
          switchMap(() => this.auth.isAuthenticated$),
          tap(isAuthenticated => {
            if(!isAuthenticated) {
              this.navCtrl.navigateRoot(PAGES.landing_page)
            }
          }), 
        );
        */
  }

  canLoad(route: Route, segments: UrlSegment[]): boolean | UrlTree | Observable<boolean | UrlTree> | Promise<boolean | UrlTree> {
    return this.authenticationService.isLoggedIn().then((isLoggedIn) => {
      if (!isLoggedIn) {
        this.navCtrl.navigateRoot(PAGES.landing_page)
        return false;
      }
      return true;
    })
  }
}


