import { HttpHandler } from '@angular/common/http';
import { HttpErrorResponse, HttpInterceptor, HttpRequest } from '@angular/common/http/';
import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { ModalController } from '@ionic/angular';
import { EMPTY, from, throwError } from 'rxjs';
import { catchError, switchMap } from 'rxjs/operators';
import { TokenResponseIdp } from 'src/app/models';
import { NoConnectionPage } from 'src/app/pages/manutenzione/no-connection/no-connection.page';
import { IdpInformationService, StorageService } from 'src/app/services';
import { TOKEN_EXCLUDE } from 'src/app/utility/global-constant';

export const http_header = 'Authorization';

@Injectable()
export class MacsHttpInterceptor implements HttpInterceptor {
	constructor(private storageService: StorageService, public router: Router, private modalController: ModalController, private idpInformationService: IdpInformationService) { }

	intercept(request: HttpRequest<any>, next: HttpHandler) {
		return from(this.storageService.getConnectionStatus()).pipe(
			switchMap((status) => {
				if (status) {
					if (!this.checkIsRequestAddToken(request.url)) {
						return from(this.storageService.getTokenIdp()).pipe(this.tokenHandler(request, next));
					} else {
						return next.handle(request);
					}
				} else {
					this.openNoConnectionModal();
					return EMPTY;
				}
			})
		);
		// return next.handle(request);
	}

	tokenHandler = (request: HttpRequest<any>, next: HttpHandler) =>
		switchMap((token: TokenResponseIdp) => {
			if (token) {
				return next.handle(this.addAuthenticationToken(request, token.accessToken)).pipe(
					catchError((error) => {
						if (error instanceof HttpErrorResponse && error.status === 401) {
							return this.handle401Error(request, next);
						} else {
							return throwError(error);
						}
					})
				);
			} else {
				return this.idpInformationService.generateSilentToken().pipe(
					switchMap((token) => {
						return next.handle(this.addAuthenticationToken(request, token['accessToken'] as string)).pipe(
							catchError((error) => {
								if (error instanceof HttpErrorResponse && error.status === 401) {
									return this.handle401Error(request, next);
								} else {
									return throwError(error);
								}
							})
						)
					})
				);
			}
			// return throwError('It was not possible to inject the token at the app startup!');
		});

	checkIsRequestAddToken(url: string): boolean {
		return TOKEN_EXCLUDE.some((item) => url.includes(item));
	}

	/*   checkIsRequestAddToken(url: string): boolean {
	 return TOKEN_EXCLUDE.filter((item) => item.includes(url)).length > 0
	} */

	addAuthenticationToken(request: HttpRequest<any>, token: string): HttpRequest<any> {
		return request.clone({
			headers: request.headers.set('Authorization', 'Bearer ' + token),
		});
	}

	async openNoConnectionModal() {
		const modal = await this.modalController.create({
			component: NoConnectionPage,
			showBackdrop: true,
		});
		return await modal.present();
	}

	handle401Error(request: HttpRequest<any>, next: HttpHandler) {
		return this.idpInformationService.getRefreshedTokenObservable().pipe(
			switchMap((token) => {
				return next.handle(this.addAuthenticationToken(request, token.accessToken));
			})
		);
	}
}
