import { Location } from '@angular/common';
import { Component, Input, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { InAppBrowser } from '@ionic-native/in-app-browser/ngx';
import { AlertButton, ModalController, Platform } from '@ionic/angular';
import { pluck } from 'rxjs/operators';
import { ModalPdfViewerComponent } from 'src/app/components';
import { DocumentGp, DownloadFileRqGp, InfoUserProcedure, PToDetailProcedure, ProcedureDocumentProtocol, ProcedureStatus, ProcedureTemplateConfig, TaskGp, UserInfo, VariableGp } from 'src/app/models';
import { GlobalService, StorageService } from 'src/app/services';
import { GestorePraticheService } from 'src/app/services/gestore-pratiche/gestore-pratiche.service';
import { OPEN_WITH_EXTERNAL_BROWSER } from 'src/app/utility';
import { DURATIONTOAST, GPFORM, GP_STATUS, IMAGE, PAGES, TEMPLATE_CONFIGURATION_DEFAULT } from 'src/app/utility/global-constant';

@Component({
    selector: 'app-dettaglio-procedura',
    templateUrl: './dettaglio-procedura.page.html',
    styleUrls: ['./dettaglio-procedura.page.scss'],
})
export class DettaglioProceduraPage implements OnInit {
    @Input() procedure: InfoUserProcedure;
    titleHeader: string = PAGES.dettaglioProcedure.title;
    logoHeader: string = IMAGE.logoHeader;
    widhtLogo: number = IMAGE.widthLogo;
    heightLogo: number = IMAGE.heightLogo;
    ios: boolean;
    android: boolean;
    procSel: string = GPFORM.procUteSel;
    listaTaskGp: Array<TaskGp>;
    variablesGp: Array<VariableGp>;
    listaDocumentGp: Array<DocumentGp>;
    showSkeletonTasksGp: boolean = true;
    showSkeletonFileGp: boolean = true;
    showSkeletonVariablesGp: boolean = true;
    showSkeletonTitleProcedureGp: boolean = true;
    showSkeletonInfoProcedureGp: boolean = true;
    showSkeletonReqProcedureGp: boolean = true;
    showSkeletonDCatastaliProcedureGp: boolean = true;
    showSkeletonOwnerProcedureGp: boolean = true;
    showSkeletonUbLavoriProcedureGp: boolean = true;
    showSkeletonIterProcedureGp: boolean = true;
    isOpenedInModal: boolean;
    hasParams: boolean;
    identifierRequester: string;
    idProcedure: number;
    userInfo: UserInfo;
    templatesConfiguration: ProcedureTemplateConfig[] = [];
    templateConfiguration: ProcedureTemplateConfig;
    procedureStatus: ProcedureStatus[] = [];
    procedureDocumentsProtocol: ProcedureDocumentProtocol[] = [];

    constructor(
        private platform: Platform,
        public location: Location,
        private globalService: GlobalService,
        private inAppBrowser: InAppBrowser,
        private storageService: StorageService,
        private gestorePraticheService: GestorePraticheService,
        private modalController: ModalController,
        private activatedRoute: ActivatedRoute) {
        this.checkPlatform();
        this.setUtility();
        this.getUserInfo();
        this.getParams();
        this.getProcedureStatus();
    }

    ngOnInit() {
        this.getUserInfo().then(() => {
            if (!this.hasParams) {
                this.isOpenedInModal = true;
                this.callServices();
            }
        });
    }

    getParams() {
        this.activatedRoute.params.pipe(pluck('params')).subscribe((result) => {
            let pToDetailProcedure: PToDetailProcedure;
            pToDetailProcedure = this.globalService.decodeParams(result);
            this.procedure = pToDetailProcedure.procedure;
            this.templatesConfiguration = pToDetailProcedure.templatesConfig;

            if (this.procedure && this.procedure?.id) {
                this.hasParams = true;
                this.callServices();
            }
        });
    }

    getProcedureStatus() {
        this.storageService.getProcedureStatus().then(
            (procedureStatus) => {
                if (procedureStatus) {
                    this.procedureStatus = procedureStatus;
                }
            }
        );
    }

    checkPlatform() {
        this.ios = this.platform.is('ios');
        this.android = this.platform.is('android');
    }

    setUtility() {
        this.listaTaskGp = [];
        this.listaDocumentGp = [];
        this.variablesGp = [];
    }

    getUserInfo() {
        return this.storageService.getUserInfo().then((userInfo) => (this.userInfo = userInfo));
    }

    callServices() {
        this.getProcuderesVariables();
        this.getTaskByUser();
        this.getInfoProcedure();
    }

    getProcuderesVariables() {
        this.gestorePraticheService.getProcuderesVariables(this.procedure?.id).subscribe(
            (result) => {
                this.showSkeletonVariablesGp = false;
                if (result.status === GP_STATUS.status_ok) {
                    this.variablesGp = result.item;
                    this.showSkeletonReqProcedureGp = false;
                    this.showSkeletonDCatastaliProcedureGp = false;
                    this.showSkeletonOwnerProcedureGp = false;
                    this.showSkeletonUbLavoriProcedureGp = false;
                    this.showSkeletonIterProcedureGp = false;
                }
            },
            () => {
                this.showSkeletonVariablesGp = false; 
                this.showSkeletonReqProcedureGp = false;
                this.showSkeletonDCatastaliProcedureGp = false;
                this.showSkeletonOwnerProcedureGp = false;
                this.showSkeletonUbLavoriProcedureGp = false;
                this.showSkeletonIterProcedureGp = false;
                this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
            }
        );
    }

    getDocumentByProcedure() {
        this.gestorePraticheService.getDocumentByProcedure(this.procedure?.id).subscribe(
            (result) => {
                this.showSkeletonFileGp = false;
                if (result.status === GP_STATUS.status_ok) {
                    if (result.item.results.length > 0) {
                        this.listaDocumentGp = result.item.results;
                        this.procedureDocumentsProtocol = this.obtainDocument(this.procedure);
                        this.listaDocumentGp = this.setDocuments(this.procedureDocumentsProtocol, this.listaDocumentGp);
                    }
                }
            },
            () => {
                this.showSkeletonFileGp = false;
                this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
            }
        );
    }

    getInfoProcedure() {
        this.gestorePraticheService.getProcedureById(this.procedure?.id).subscribe(
            (result) => {
                this.procedure = result.item;
                if (result.status == 'OK' && this.procedure) {
                    this.templateConfiguration = this.getCodeTemplatePratica(this.templatesConfiguration, this.procedure.procedureId) ?
                        this.getCodeTemplatePratica(this.templatesConfiguration, this.procedure.procedureId) : TEMPLATE_CONFIGURATION_DEFAULT;

                    // Recupero i documenti 
                    this.getDocumentByProcedure();
                    this.showSkeletonTitleProcedureGp = false;
                    this.showSkeletonInfoProcedureGp = false;
                } else {
                }
            },
            () => {
                this.showSkeletonFileGp = false;
                this.showSkeletonInfoProcedureGp = false;
                this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
            }
        );
    }

    getTaskByUser() {
        this.gestorePraticheService.getTaskByUser(this.procedure?.requester?.identifier, this.procedure?.id).subscribe(
            (result) => {
                this.showSkeletonTasksGp = false;
                if (result.status === GP_STATUS.status_ok) {
                    if ((result.item as unknown as TaskGp[]).length > 0) {
                        this.listaTaskGp = result.item as unknown as TaskGp[];
                    }
                }
            },
            () => {
                this.showSkeletonTasksGp = false;
                this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
            }
        );
    }

    downloadDocumentGp(file: DocumentGp) {
        this.gestorePraticheService.downloadDocumentGp(this.prepareRequestDownFile(file.id)).subscribe(
            (result) => {
                if (file.mimetype.includes('pdf')) {
                    this.openPdfViewrModal(result, file.name);
                } else {
                    this.confirmDownloadFile(file.name, result);
                }
            },
            () => this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default)
        );
    }

    downloadFile(file: DocumentGp) {
        /*   this.openUrl(this.globalService.b64toBlob(IMAGE_FITTIZIA.base_64, 'image/png')) */
        this.downloadDocumentGp(file);
    }

    openUrl(file: Blob) {
        const url = window.URL.createObjectURL(file);
        this.inAppBrowser.create(url, OPEN_WITH_EXTERNAL_BROWSER ? '_system' : '_blank', 'hideurlbar=yes,location=no,toolbar=no,enableViewportScale=yes');
    }

    prepareRequestDownFile(idDocument: string): DownloadFileRqGp {
        return {
            documentId: idDocument,
            requesterIdentifier: this.procedure.requester.identifier,
        } as DownloadFileRqGp;
    }

    closeModalDetail() {
        this.modalController.dismiss();
    }

    async openPdfViewrModal(pdf: Blob, fileName: string) {
        let fileURL = URL.createObjectURL(pdf);
        const modal = await this.modalController.create({
            component: ModalPdfViewerComponent,
            showBackdrop: true,
            componentProps: {
                pdfSrc: fileURL,
                base64File: pdf,
                fileName: fileName,
            },
        });
        return await modal.present();
    }

    confirmDownloadFile(fileName: string, fileBase64: Blob) {
        const buttons: AlertButton[] = [
            {
                text: 'No',
                handler: () => {
                },
            },
            {
                text: 'Si',
                cssClass: 'secondary',
                handler: () => {
                    this.globalService
                        .downloadFile(fileName, fileBase64)
                        .then((res) => {
                            // this.globalService.presentToast('Il file è stato salvato nella cartella Download del dispositivo', DURATIONTOAST.default);
                            this.globalService.openFile(res.nativeURL);
                        })
                        .catch((err) => {
                            this.globalService.presentToast('Impossibile scaricare il file', DURATIONTOAST.default);
                        });
                },
            },
        ];
        this.globalService.presentAlertConfirm('Conferma Download', 'Si desidera scaricare il file ?', buttons);
    }

    getCodeTemplatePratica(templatesConfig: ProcedureTemplateConfig[], idProcedure: number): ProcedureTemplateConfig {
        return templatesConfig.length === 0 ? null : templatesConfig.find((template) => Number(template.idProcedure) === idProcedure);
    }

    /**
    * Ottenere i documenti con il numero di protocollo contenente solo numeri
    * 
    * 
    * @param {InfoUserProcedure} procedure Procedure
    * @returns I documenti
    */
    obtainDocument(procedure: InfoUserProcedure): ProcedureDocumentProtocol[] {
        return procedure.documentProtocolModelList.filter(pdp => Number.isInteger(Number(pdp.protocolNumber)));
    }

    /**
    * Filtra i documenti in base in base ai documenti recuperati con la logica del numero protocollo numerico
    * 
    * 
    * @param {ProcedureDocumentProtocol[]} documents Documenti parziali
    * @param {DocumentGp[]} documents Documenti totali
    * @returns I documenti
    */
    setDocuments(procedureDocumentsProtocol: ProcedureDocumentProtocol[], documents: DocumentGp[]): DocumentGp[] {
        let documentsID: string[] = [];
        procedureDocumentsProtocol.forEach(
            pdp => {
                pdp.documentIdList.forEach(doc => documentsID.push(doc))
            }
        );
        return documents.filter((doc) => documentsID.find(idDoc => idDoc === doc.id));

    }
}
