import { HttpClient } from '@angular/common/http';
import { Component, ElementRef, Input, OnInit, Renderer2, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { DatePipe, Location } from '@angular/common';
import { InAppBrowser } from '@ionic-native/in-app-browser/ngx';
import { ModalController, Platform } from '@ionic/angular';
import { pluck } from 'rxjs/operators';
import { ModalPdfViewerComponent } from 'src/app/components';
import { Allegato, News, NewsEventiDettaglio } from 'src/app/models';
import { GlobalService, NewsEventiService } from 'src/app/services';
import { htmlCodeToChar, OPEN_WITH_EXTERNAL_BROWSER } from 'src/app/utility';
import { DURATIONTOAST, IMAGE, NEWS_DETAIL_BODY_MAX_HEIGHT, PAGES, TIPNEWS, VIDEOPLAYER } from 'src/app/utility/global-constant';
import { CONFIGSLIDE } from 'src/app/utility/slider.config';
import { ManagePushService } from '../../../services/manage-push/manage-push.service';
import { normalizeImgTag } from '../../../utility/character';
import { DETAIL_NEWS_LABELS, getExtensionFromMediaType } from '../../../utility/global-constant';
import { GalleriaNewsPage } from '../galleria-news/galleria-news.page';

@Component({
    selector: 'app-dettaglio-news',
    templateUrl: './dettaglio-news.page.html',
    styleUrls: ['./dettaglio-news.page.scss'],
})
export class DettaglioNewsPage implements OnInit {
    @Input() idNe: string;
    @Input() isDrawable: string;
    @ViewChild('newsContent') newsContent: ElementRef<HTMLDivElement>;
    titleHeader: string = PAGES.dettaglioNews.title;
    logoHeader: string = IMAGE.logoHeader;
    widhtLogo: number = IMAGE.widthLogo;
    heightLogo: number = IMAGE.heightLogo;
    neDettaglio: NewsEventiDettaglio;
    news: News;
    slideOpts = CONFIGSLIDE;
    widthVideoPlayer: number = VIDEOPLAYER.width;
    heightVideoPlayer: number = VIDEOPLAYER.height;
    ios: boolean;
    android: boolean;
    isBackdropActive: boolean = false;
    showSkeleton: boolean = true;
    sizeRowBodySkeleton: number = 15;
    isOpenedInModal: boolean;
    hasParams: boolean;
    imageTemp: string = IMAGE.neLocandinaTemp;
    titleCategory: string;
    showBodyHtml: boolean = false;
    showAttachment: boolean = false;
    showRelated: boolean = false;
    detailNewsLabels = DETAIL_NEWS_LABELS;
    showAltImage: boolean = false;
    NEWS_DETAIL_BODY_MAX_HEIGHT = NEWS_DETAIL_BODY_MAX_HEIGHT;
    isIos: boolean = false;
    isLoadedImage: boolean = false;
    isEvento: boolean = false;
    firstTextOrgDa: string = '<';
    secondTextOrgDa: string = '>';
    
    constructor(
        private platform: Platform,
        private globalService: GlobalService,
        private modalController: ModalController,
        private newsEventiService: NewsEventiService,
        private inAppBrowser: InAppBrowser,
        private activatedRoute: ActivatedRoute,
        private managePushService: ManagePushService,
        private http: HttpClient,
        private renderer: Renderer2,
        private location: Location,
        private datePipe: DatePipe
    ) {
        this.checkPlatform();
        this.getParams();
        this.isIos = platform.is('ios');
    }

    ngOnInit() {
        if (!this.hasParams) {
            this.getDettaglioNPCA(this.idNe);
            this.isOpenedInModal = true;
        }
    }

    ionViewWillLeave() {
        this.globalService.stopNgxLoader();
    }

    getParams() {
        this.activatedRoute.params.pipe(pluck('params')).subscribe((result) => {
            this.idNe = this.globalService.decodeParams(result)['id_ne'];
            const titleCategory = this.globalService.decodeParams(result)['titleCategory'];
            if (titleCategory) {
                this.titleCategory = titleCategory;
            }
            if (this.idNe) {
                this.hasParams = true;
                this.titleCategory.toLocaleUpperCase() === TIPNEWS[4].t_nome_dettagio.toLocaleUpperCase() ? this.getDettaglioEvento(this.idNe) : this.getDettaglioNPCA(this.idNe);
            } else {
                this.location.back();
            }
        });
    }

    checkPlatform() {
        this.ios = this.platform.is('ios');
        this.android = this.platform.is('android');
    }

    setUtility() { }

    getDettaglioNPCA(id_ne: string) {
        this.newsEventiService.getDettaglioNPCA(id_ne).subscribe(
            (result) => {
                this.showSkeleton = false;
                this.neDettaglio = result?.rows[0];
                this.neDettaglio.title = normalizeImgTag(htmlCodeToChar(this.neDettaglio.title), result?.rows[0].base_url);
                this.neDettaglio.field_descrizione_abstract = normalizeImgTag(
                    htmlCodeToChar(this.neDettaglio.field_descrizione_abstract),
                    result?.rows[0].base_url
                );
                this.neDettaglio.body = normalizeImgTag(htmlCodeToChar(this.neDettaglio.body), result?.rows[0].base_url);
                this.handleLinkTag();
            },
            (err) => {
                this.showSkeleton = false;
                this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
            }
        );
    }

    viewAllegato(allegato: Allegato, openModalNews: boolean = false) {
        // allegato.url = allegato.url.replace('test-', '');
        if (allegato.id && openModalNews) {
            this.openModalDetailNews(allegato.id);
        } else {
            this.globalService.startNgxLoader();
            fetch(allegato.url)
                .then((res: Response) => {
                    return res.blob();
                })
                .then((blob) => {
                    if (allegato.url?.indexOf('.pdf') > 0) {
                        this.openPdfViewrModal(allegato.url, allegato.nome + '.pdf', blob);
                    } else {
                        const extension = getExtensionFromMediaType(blob.type);
                        if (['.html', '.htm'].includes(extension)) {
                            this.inAppBrowser.create(
                                allegato.url,
                                OPEN_WITH_EXTERNAL_BROWSER ? '_system' : '_blank',
                                'hideurlbar=yes,location=no,toolbar=no,enableViewportScale=yes'
                            );
                        } else {
                            this.globalService
                                .downloadFile(`${allegato.nome}${extension}`, blob)
                                .then((res) => {
                                    this.globalService.openFile(res.nativeURL);
                                })
                                .catch((err) => {
                                    this.globalService.presentToast('Impossibile scaricare il file', DURATIONTOAST.default);
                                });
                        }
                    }
                })
                .catch((err) => {
                    this.inAppBrowser.create(
                        allegato.url,
                        OPEN_WITH_EXTERNAL_BROWSER ? '_system' : '_blank',
                        'hideurlbar=yes,location=no,toolbar=no,enableViewportScale=yes'
                    );
                })
                .finally(() => {
                    this.globalService.stopNgxLoader();
                });
        }
    }

    vediGalleria(isImageGallery: boolean) {
        this.apriModalGalleria(isImageGallery);
    }

    async apriModalGalleria(isImageGallery: boolean) {
        const modal = await this.modalController.create({
            component: GalleriaNewsPage,
            cssClass: 'modal-galleriane',
            showBackdrop: true,
            componentProps: {
                listaImage: this.neDettaglio.listaImage,
                listaVideoLocali: this.neDettaglio.listaVideoLocali,
                listaVideoEsterni: this.neDettaglio.listaVideoEsterni,
                isImageGallery,
            },
        });
        this.isBackdropActive = true;
        modal.onWillDismiss().then(() => {
            this.isBackdropActive = false;
        });
        return await modal.present();
    }

    doRefresh(event) {
        this.getDettaglioNPCA(this.idNe);
        setTimeout(() => {
            event.target.complete();
        }, 250);
    }

    closeModalDetail() {
        this.modalController.dismiss();
    }

    async openPdfViewrModal(pdf: string, fileName: string, file: Blob) {
        const modal = await this.modalController.create({
            component: ModalPdfViewerComponent,
            showBackdrop: true,
            componentProps: {
                pdfSrc: pdf,
                base64File: file,
                fileName,
            },
        });
        return await modal.present();
    }

    toggleRow(rowCase: number) {
        switch (rowCase) {
            case 1:
                this.showBodyHtml = !this.showBodyHtml;
                break;
            case 2:
                this.showAttachment = !this.showAttachment;
                break;
            case 3:
                this.showRelated = !this.showRelated;
                break;
            default:
                console.log('case non trovato');
        }
    }

    loadAlternativeImage() {
        this.showAltImage = true;
        this.isLoadedImage = true;
    }

    getFileName(allegato: Allegato) {
        if (!allegato.nome) {
            const splittedUrl = allegato.url.split('/');
            if (splittedUrl && splittedUrl.length !== 0) {
                return splittedUrl[splittedUrl.length - 1];
            }
        }

        return allegato.nome;
    }

    dynamicHeightStyle(textBody: string) {
        if (textBody) {
            const height = textBody.length <= NEWS_DETAIL_BODY_MAX_HEIGHT ? textBody.length : NEWS_DETAIL_BODY_MAX_HEIGHT;
            return this.showBodyHtml ? '' : `height:${height}px`;
        } else {
            return '';
        }
    }

    async openModalDetailNews(idNews: string) {
        const modal = await this.modalController.create({
            component: DettaglioNewsPage,
            componentProps: {
                idNe: idNews,
                isDrawable: true,
            },
            showBackdrop: true,
            keyboardClose: false,
            cssClass: 'custom-news-modal',
            animated: true,
            swipeToClose: true,
            mode: 'ios',
        });
        return await modal.present();
    }

    loadedImage() {
        this.isLoadedImage = true;
    }

    handleLinkTag() {
        setTimeout(() => {
            const links: HTMLCollection = this.newsContent.nativeElement.getElementsByTagName('a');
            for (let i = 0; i < links.length; i++) {
                this.renderer.listen(links[i], 'click', (e) => {
                    e.preventDefault();
                    const href = links[i].getAttribute('href');
                    this.viewAllegato({ url: href, nome: 'link' });
                });
            }
        }, 1000);
    }

    getDettaglioEvento(id_evento: string) {
        this.isEvento = true;
        this.newsEventiService.getDettaglioEvento(id_evento).subscribe(
            (result) => {
                this.showSkeleton = false;
                this.neDettaglio = result?.data;
                this.neDettaglio.title = normalizeImgTag(htmlCodeToChar(this.neDettaglio.title), result?.data.base_url);
                this.neDettaglio.field_descrizione_abstract = normalizeImgTag(
                    htmlCodeToChar(this.neDettaglio.field_descrizione_abstract),
                    result?.data.base_url
                );
                this.neDettaglio.body = normalizeImgTag(htmlCodeToChar(this.neDettaglio.body), result?.data.base_url);
                this.handleLinkTag();
            },
            (err) => {
                this.showSkeleton = false;
                this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
            }
        );
    }

    transformDate(value: string) {
        try {
            return this.datePipe.transform(value, 'dd/MM/yyyy');
        } catch (e) {
            return value ? value : '';
        }
    }
}
