import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { IonContent, Platform } from '@ionic/angular';
import { forkJoin } from 'rxjs';
import { NewsEventiDettaglio, NewsEventiRequest, NewsTipologia } from 'src/app/models';
import { GlobalService, NewsEventiService } from 'src/app/services';
import { htmlCodeToChar } from 'src/app/utility';
import { DURATIONTOAST, IMAGE, MESSAGES, PAGES, PAGINATION_NE, SEARCH, TIPNEWS } from 'src/app/utility/global-constant';

@Component({
	selector: 'app-news',
	templateUrl: './news.page.html',
	styleUrls: ['./news.page.scss'],
})
export class NewsPage implements OnInit {
	@ViewChild(IonContent) content: IonContent;
	titleHeader: string = PAGES.news.title;
	logoHeader: string = IMAGE.logoHeader;
	widhtLogo: number = IMAGE.widthLogo;
	heightLogo: number = IMAGE.heightLogo;
	listaNewsEventi: Array<NewsEventiDettaglio>;
	listaNewsEventiPlus: Array<NewsEventiDettaglio>;
	listaTipNews: Array<NewsTipologia>;
	imageTemp: string = IMAGE.neLocandinaTemp;
	notResult: boolean = false;
	textMessage: string = MESSAGES.notNews;
	debounceSearch: number = SEARCH.debounce;
	page_number: number;
	neFinished: boolean;
	catScelected: string;
	ios: boolean;
	android: boolean;
	titleSearched: string;
	showSkeleton: boolean = true;
	sizeSkeleton: number = 4;
	isEventi: boolean; 

	constructor(
		private platform: Platform,
		private globalService: GlobalService,
		private newsEventiService: NewsEventiService,
		private router: Router,
		private activatedRoute: ActivatedRoute) {
		this.checkPlatform();
		this.setUtility();
		this.callServiceByTipologia(false, true);
	}

	ngOnInit() { }

	setUtility() {
		this.listaNewsEventi = [];
		this.listaNewsEventiPlus = [];
		this.setInitPagination();
		this.titleSearched = '';
		this.listaTipNews = TIPNEWS;
		this.catScelected = this.listaTipNews[0].t_id_tipologia;
	}

	checkPlatform() {
		this.ios = this.platform.is('ios');
		this.android = this.platform.is('android');
	}

	detailsNE(newsEventi: NewsEventiDettaglio) {
		this.goToDetailNews(newsEventi);
	}

	doInfinite(event) {
		if (!this.neFinished && !this.showSkeleton) {
			this.callServiceByTipologia(true);
			// setTimeout(() => {
			// this.upgradeListaNewsEventi(this.listaNewsEventiPlus);
			event.target.complete();
			// }, 1000);
		} else {
			event.target.complete();
		}
	}

	upgradeListaNewsEventi(listaNewsEventiPlus: Array<NewsEventiDettaglio>) {
		for (let news of listaNewsEventiPlus) {
			this.listaNewsEventi.push(news);
		}
		this.listaNewsEventiPlus = [];
	}

	doRefresh(event) {
		this.setInitPagination();
		this.callServiceByTipologia(false, true);
		setTimeout(() => {
			event.target.complete();
		}, 250);
	}

	setInitPagination() {
		this.page_number = PAGINATION_NE.start_page;
		this.neFinished = false;
	}

	setEndPagination() {
		this.neFinished = true;
	}

	setCardMessage(zeroResult: boolean) {
		if (zeroResult) {
			this.notResult = true;
			this.listaNewsEventi = [];
		}
	}

	searchNEByTitle(evento: Event) {
		this.setInitPagination();
		this.titleSearched = evento['detail']['value'] as string;
		this.forceContent();
		this.callServiceByTipologia();
		return true;
	}

	searchNEByTipologia(evento: Event) {
		this.catScelected = evento['detail']['value'] as string;
		this.titleSearched = '';
		this.setInitPagination();
		this.forceContent();
		this.callServiceByTipologia(false, true);
		return true;
	}

	goToDetailNews(newsEventiDettaglio: NewsEventiDettaglio) {
		let catSelectendName = '';
		if (newsEventiDettaglio.field_tipologia_notizia) {
			catSelectendName = newsEventiDettaglio.field_tipologia_notizia;
			if (catSelectendName.toLowerCase() === 'news') {
				catSelectendName = 'Notizia';
			}
		} else {
			catSelectendName = this.listaTipNews.find((ltn) => ltn.t_id_tipologia === this.catScelected)?.t_nome_dettagio;
		}
		let params = this.globalService.encodeParams({
			id_ne: newsEventiDettaglio.nid ? newsEventiDettaglio.nid : newsEventiDettaglio.id,
			titleCategory: catSelectendName,
		});
		this.router.navigate([PAGES.dettaglioNews.link, params], { relativeTo: this.activatedRoute });
	}

	callServiceByTipologia(plusElements?: boolean, firstTime?: boolean) {
		this.isEventi = false;
		switch (this.catScelected) {
			case TIPNEWS[0].t_id_tipologia:
				firstTime ? this.getNewsPrimoPiano() : this.getPrimoPianoPaginated(plusElements);
				break;
			case TIPNEWS[1].t_id_tipologia:
				this.getNewsPaginated(plusElements);
				break;
			case TIPNEWS[2].t_id_tipologia:
				this.getComunicatiPaginated(plusElements);
				break;
			case TIPNEWS[3].t_id_tipologia:
				this.getAvvisiPaginated(plusElements);
				break;
			case TIPNEWS[4].t_id_tipologia:
				this.isEventi = true;
				this.getEventiPaginated(plusElements);
				break;
		}
	}

	prepareRequestNE(): NewsEventiRequest {
		return {
			items_per_page: PAGINATION_NE.items_per_page,
			page: this.page_number,
			title: this.titleSearched,
		} as NewsEventiRequest;
	}

	getPrimoPianoPaginated(plusElements?: boolean) {
		plusElements ? (this.page_number += 1) : null;
		this.newsEventiService.getPrimoPianoPaginated(this.prepareRequestNE()).subscribe({
			next: (result) => {
				this.showSkeleton = false;
				if (result.rows.length > 0) {
					this.normalizeRows(result.rows);
					plusElements ? (this.listaNewsEventiPlus = result.rows) : (this.listaNewsEventi = result.rows);
				} else {
					plusElements ? this.setEndPagination() : this.setCardMessage(true);
				}
			},
			error: (error) => {
				this.showSkeleton = false;
				this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
				this.setEndPagination();
				this.setCardMessage(true);
			},
			complete: () => {
				this.upgradeListaNewsEventi(this.listaNewsEventiPlus);
			},
		});
	}

	getNewsPaginated(plusElements?: boolean) {
		plusElements ? (this.page_number += 1) : null;
		this.newsEventiService.getNewsPaginated(this.prepareRequestNE()).subscribe({
			next: (result) => {
				this.showSkeleton = false;
				if (result.rows.length > 0) {
					this.normalizeRows(result.rows);
					plusElements ? (this.listaNewsEventiPlus = result.rows) : (this.listaNewsEventi = result.rows);
				} else {
					plusElements ? this.setEndPagination : this.setCardMessage(true);
				}
			},
			error: (error) => {
				this.showSkeleton = false;
				this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
				this.setEndPagination();
				this.setCardMessage(true);
			},
			complete: () => {
				this.upgradeListaNewsEventi(this.listaNewsEventiPlus);
			},
		});
	}

	getComunicatiPaginated(plusElements?: boolean) {
		plusElements ? (this.page_number += 1) : null;
		this.newsEventiService.getComunicatiPaginated(this.prepareRequestNE()).subscribe({
			next: (result) => {
				this.showSkeleton = false;
				if (result.rows.length > 0) {
					this.normalizeRows(result.rows);
					plusElements ? (this.listaNewsEventiPlus = result.rows) : (this.listaNewsEventi = result.rows);
				} else {
					plusElements ? this.setEndPagination : this.setCardMessage(true);
				}
			},
			error: (error) => {
				this.showSkeleton = false;
				this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
				this.setEndPagination();
				this.setCardMessage(true);
			},
			complete: () => {
				this.upgradeListaNewsEventi(this.listaNewsEventiPlus);
			},
		});
	}

	getAvvisiPaginated(plusElements?: boolean) {
		plusElements ? (this.page_number += 1) : null;
		this.newsEventiService.getAvvisiPaginated(this.prepareRequestNE()).subscribe({
			next: (result) => {
				this.showSkeleton = false;
				if (result.rows.length > 0) {
					this.normalizeRows(result.rows);
					plusElements ? (this.listaNewsEventiPlus = result.rows) : (this.listaNewsEventi = result.rows);
				} else {
					plusElements ? this.setEndPagination : this.setCardMessage(true);
				}
			},
			error: (error) => {
				this.showSkeleton = false;
				this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
				this.setEndPagination();
				this.setCardMessage(true);
			},
			complete: () => {
				this.upgradeListaNewsEventi(this.listaNewsEventiPlus);
			},
		});
	}

	getEventiPaginated(plusElements?: boolean) {
		plusElements ? (this.page_number += 1) : null;
		this.newsEventiService.getEventiPaginated(this.prepareRequestNE()).subscribe({
			next: (result) => {
				this.showSkeleton = false;
				if (result.data.length > 0) {
					this.normalizeRows(result.data);
					plusElements ? (this.listaNewsEventiPlus = result.data) : (this.listaNewsEventi = result.data);
				} else {
					plusElements ? this.setEndPagination : this.setCardMessage(true);
				}
			},
			error: (error) => {
				this.showSkeleton = false;
				this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
				this.setEndPagination();
				this.setCardMessage(true);
			},
			complete: () => {
				this.upgradeListaNewsEventi(this.listaNewsEventiPlus);
			},
		});
	}

	forceContent() {
		//this.content.forceOverscroll = true;
		this.content.scrollToTop();
	}

	normalizeRows(newsEventList: NewsEventiDettaglio[]) {
		newsEventList.forEach((ne) => {
			ne.title = htmlCodeToChar(ne.title);
		});
	}

	getNewsPrimoPiano() {
		forkJoin([
			this.newsEventiService.getPrimoPianoHome(),
			this.newsEventiService.getPrimoPianoPaginated(this.prepareRequestNE())
		])
			.subscribe({
				next: (results) => {
					this.showSkeleton = false;
					this.listaNewsEventi = [];
					if (results[0] && results[1]) {
						if (results[0].rows && results[0].rows.length > 0) {
							this.normalizeRows(results[0].rows);
							this.listaNewsEventi = [...this.listaNewsEventi, ...results[0].rows];
						}
						if (results[1].rows && results[1].rows.length > 0) {
							this.normalizeRows(results[1].rows);
							this.listaNewsEventi = [...this.listaNewsEventi, ...results[1].rows];
						}
					}
				},
				error: () => {
					this.showSkeleton = false;
					this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
					this.setEndPagination();
					this.setCardMessage(true);
				},
				complete: () => {

				},
			});
	}
}
