import { Component, OnInit, ViewChild } from '@angular/core';
import { ModalController, Platform } from '@ionic/angular';
import { Map } from 'leaflet';
import { PlaceAddress, PlacePortale } from 'src/app/models';
import { StorageService } from 'src/app/services';
import { DETAIL_PLACE_LABELS, IMAGE, PAGES, PLACE_DETAIL_BODY_MAX_HEIGHT } from 'src/app/utility';
import { GalleriaNewsPage } from '../../news-eventi/galleria-news/galleria-news.page';

@Component({
  selector: 'app-poi-dettaglio',
  templateUrl: './poi-dettaglio.page.html',
  styleUrls: ['./poi-dettaglio.page.scss'],
})
export class PoiDettaglioPage implements OnInit {
  @ViewChild('map') map: Map;
  titleHeader: string = PAGES.dettaglioLuogo.title;
  logoHeader: string = IMAGE.logoHeader;
  widhtLogo: number = IMAGE.widthLogo;
  heightLogo: number = IMAGE.heightLogo;
  ios: boolean;
  android: boolean;
  showSkeletonDetail: boolean = true;
  showSkeletonMap: boolean = true;
  place: PlacePortale;
  isLoadedImage: boolean = false;
  showAltImage: boolean = false;
  imageTemp: string = IMAGE.neLocandinaTemp;
  isBackdropActive: boolean = false;
  showBodyHtml: boolean = false;
  showAttachment: boolean = false;
  showRelated: boolean = false;
  detailNewsLabels = DETAIL_PLACE_LABELS;
  PLACE_DETAIL_BODY_MAX_HEIGHT = PLACE_DETAIL_BODY_MAX_HEIGHT;
  placeAddress: PlaceAddress;

  constructor(
    private platform: Platform,
    private modalController: ModalController,
    private storageService: StorageService) {
    this.getPlaceFromStore();
  }

  ngOnInit() {
  }

  checkPlatform() {
    this.ios = this.platform.is('ios');
    this.android = this.platform.is('android');
  }

  async getPlaceFromStore() {
    this.place = await this.storageService.getPlaceToShow();
    if (this.place) {
      this.showSkeletonDetail = !this.showSkeletonDetail;
      this.placeAddress = this.createAddressObj(this.place);
      this.showSkeletonMap = this.placeAddress ? false : true;
    }
  }

  loadAlternativeImage() {
    this.showAltImage = true;
    this.isLoadedImage = true;
  }

  loadedImage() {
    this.isLoadedImage = true;
  }

  vediGalleria(isImageGallery: boolean) {
    this.apriModalGalleria(isImageGallery);
  }

  async apriModalGalleria(isImageGallery: boolean) {
    const modal = await this.modalController.create({
      component: GalleriaNewsPage,
      cssClass: 'modal-galleriane',
      showBackdrop: true,
      componentProps: {
        listaImage: this.place.listaImage,
        /*    listaVideoLocali: this.neDettaglio.listaVideoLocali,
           listaVideoEsterni: this.neDettaglio.listaVideoEsterni, */
        isImageGallery,
      },
    });
    this.isBackdropActive = true;
    modal.onWillDismiss().then(() => {
      this.isBackdropActive = false;
    });
    return await modal.present();
  }

  dynamicHeightStyle(textBody: string) {
    if (textBody) {
      const height = textBody.length <= PLACE_DETAIL_BODY_MAX_HEIGHT ? textBody.length : PLACE_DETAIL_BODY_MAX_HEIGHT;
      return this.showBodyHtml ? '' : `height:${height}px`;
    } else {
      return '';
    }
  }

  toggleRow(rowCase: number) {
    switch (rowCase) {
      case 1:
        this.showBodyHtml = !this.showBodyHtml;
        break;
      case 2:
        this.showAttachment = !this.showAttachment;
        break;
      case 3:
        this.showRelated = !this.showRelated;
        break;
      default:
        console.log('case non trovato');
    }
  }

  createAddressObj(place: PlacePortale): PlaceAddress {
    return {
      field_cap: place?.field_cap,
      field_circoscrizione: place?.field_circoscrizione,
      field_indirizzo: place?.field_indirizzo,
      filed_stato: 'ITALIA'
    };
  }



}
