import { Location } from '@angular/common';
import { Component, Input, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { InAppBrowser } from '@ionic-native/in-app-browser/ngx';
import { AlertController, ModalController, NavController, Platform } from '@ionic/angular';
import { DisdiciPrenotazioneRequest, Prenotazione, PrenotazioneRequest } from 'src/app/models';
import { GlobalService, PrenotazioneService, StorageService } from 'src/app/services';
import { OPEN_WITH_EXTERNAL_BROWSER } from 'src/app/utility';
import { DURATIONTOAST, IMAGE, PAGES } from 'src/app/utility/global-constant';

import { ModalPdfViewerComponent } from '../../../components';

@Component({
    selector: 'app-dettaglio-prenotazione',
    templateUrl: './dettaglio-prenotazione.page.html',
    styleUrls: ['./dettaglio-prenotazione.page.scss'],
})
export class DettaglioPrenotazionePage implements OnInit {
    @Input() idPrenotazione: number;
    titleHeader: string = PAGES.dettaglioPrenotazione.title;
    logoHeader: string = IMAGE.logoHeader;
    widhtLogo: number = IMAGE.widthLogo;
    heightLogo: number = IMAGE.heightLogo;
    prenotazione: any;
    dpRequest: DisdiciPrenotazioneRequest;
    showSkeleton: boolean = true;
    isOpenedInModal: boolean = true;

    constructor(
        public alertController: AlertController,
        private globalService: GlobalService,
        private platform: Platform,
        private inapp: InAppBrowser,
        private router: Router,
        private activatedRoute: ActivatedRoute,
        public location: Location,
        private prenotazioneService: PrenotazioneService,
        private modalController: ModalController,
        private navCtrl: NavController,
        private storageService: StorageService
    ) {
        this.getParams();
    }

    ngOnInit() {
        if (this.idPrenotazione) {
            this.getPrenotazioneByID(this.idPrenotazione);
        } else {
            this.navCtrl.navigateRoot(PAGES.home_page);
        }
    }

    getParams() {}

    scaricaRicevuta(pdfBase64: string) {
        let blob = this.globalService.b64toBlob(pdfBase64, 'application/pdf');
        let fileURL = URL.createObjectURL(blob);
        if (this.platform.is('ios')) {
            this.inapp.create(fileURL, OPEN_WITH_EXTERNAL_BROWSER ? '_system' : '_blank', 'hideurlbar=yes,location=no,toolbar=no,enableViewportScale=yes');
        } else {
            this.inapp.create(fileURL, OPEN_WITH_EXTERNAL_BROWSER ? '_system' : '_blank', 'hideurlbar=yes,location=no,toolbar=no,enableViewportScale=yes');
        }
    }

    async confermaOrAnnullaDisdetta(prenotazione: Prenotazione) {
        const alert = await this.alertController.create({
            cssClass: 'my-custom-class',
            header: '',
            message: 'Sei sicuro di voler disdire ?',
            buttons: [
                {
                    text: 'No',
                    role: 'cancel',
                },
                {
                    text: 'Si',
                    role: 'ok',
                    handler: () => {
                        this.disdiciPrenotazione(prenotazione);
                    },
                },
            ],
        });
        await alert.present();
    }

    disdiciPrenotazione(prenotazione: Prenotazione) {
        this.dpRequest = {
            cfFruitore: prenotazione.anagraficaFruitore.cf,
            cfUser: prenotazione.anagraficaPresentante.cf,
            idPrenotazione: prenotazione.idPrenotazione,
        };
        this.prenotazioneService.disdiciPrenotazione(this.dpRequest).subscribe(
            (result) => {
                if (result.success) {
                    this.showMessagePD();
                } else {
                    this.globalService.presentToast('La prenotazione non è stata disdetta', DURATIONTOAST.default);
                }
            },
            (error) => {
                this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
            }
        );
    }

    async showMessagePD() {
        const alert = await this.alertController.create({
            cssClass: 'my-custom-class',
            header: '',
            message: 'Prenotazione disdetta',
        });
        await alert.present();
        setTimeout(() => {
            alert.dismiss();
            this.router.navigate([PAGES.home_page], { relativeTo: this.activatedRoute });
        }, 2000);
    }

    async openPdfViewrModal(pdfBase64: string) {
        let blob = this.globalService.b64toBlob(pdfBase64, 'application/pdf');
        let fileURL = URL.createObjectURL(blob);
        const modal = await this.modalController.create({
            component: ModalPdfViewerComponent,
            showBackdrop: true,
            componentProps: {
                pdfSrc: fileURL,
            },
        });
        return await modal.present();
    }

    getPrenotazioneByID(idPrenotazione: number) {
        this.storageService.getUserInfo().then((ui) => {
            if (ui) {
                let prenotazioneRequest: PrenotazioneRequest = {
                    cfUser: ui.identifier.toLocaleUpperCase(),
                    cfFruitore: ui.identifier.toLocaleUpperCase(),
                    idPrenotazione: idPrenotazione,
                };
                this.prenotazioneService.getPrenotazioneByID(prenotazioneRequest).subscribe(
                    (result) => {
                        this.showSkeleton = false;
                        if (result.success) {
                            if (result.data.content.prenotazioni && result.data.content.prenotazioni.length > 0) {
                                this.prenotazione = result.data.content.prenotazioni[0];
                            }
                        }
                    },
                    (error) => {
                        this.showSkeleton = false;
                        console.log('getPrenotazioneByID-error', error);
                    }
                );
            }
        });
    }

    closeModalDetail() {
        this.modalController.dismiss();
    }
}
