import { Injectable } from '@angular/core';
import { Device } from '@ionic-native/device/ngx';
import { Storage } from '@ionic/storage';
import { BehaviorSubject } from 'rxjs';
import { LogoutRequest } from 'src/app/models';
import { FirebaseService } from '..';
import { GestorePraticheService } from '../gestore-pratiche/gestore-pratiche.service';
import { StorageService } from '../storage/storage.service';

@Injectable({
  providedIn: 'root'
})
export class AuthenticationService {
  HAS_LOGGED_IN = 'hasLoggedIn';
  public loggedIN$: BehaviorSubject<boolean>;

  constructor(
    private storage: Storage,
    private storageService: StorageService,
    private firebaseService: FirebaseService,
    private gestorePraticheService: GestorePraticheService,
    public device: Device) {
    this.loggedIN$ = new BehaviorSubject<boolean>(false);
  }

  //RIVIDERE

  login(): Promise<any> {
    return this.storage.set(this.HAS_LOGGED_IN, true).then(() => {
      this.gestorePraticheService.generateToken();
      this.loggedIN$.next(true);
    });
  }

  logout(): Promise<any> {
    return this.storage.remove(this.HAS_LOGGED_IN).then(() => {
      this.loggedIN$.next(false);
      this.logoutDispositivo();
    })
  }

  isLoggedIn(): Promise<boolean> {
    return this.storage.get(this.HAS_LOGGED_IN).then((value) => {
      return value;
    });
  }

  logoutDispositivo() {
    this.storageService.getUserInfo().then((infoUser)=> {
      let logoutRequest: LogoutRequest;
      if (infoUser?.identifier && this.device.uuid) {
        logoutRequest = {
          spidCode: infoUser?.identifier, uuid: this.device.uuid, piattaforma: this.device.platform
        };
        this.firebaseService.logoutDispositivo(logoutRequest).subscribe(
          (result) => {
            if (!result.success)
              console.log('[AuthenticationService - logoutDispositivo - KO]');
            else
              console.log('[AuthenticationService - logoutDispositivo - OK]');
            this.removeAll();
          },
          (error) => {
            console.log('[AuthenticationService - logoutDispositivo - KO]');
            this.removeAll();
          })
      }
    });
  }

  removeAll() {
    this.storageService.removeAll();
  }

}
