import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import {
    DocumentGp,
    DownloadFileRqGp,
    FilterProcedureGp,
    InfoUserProcedure,
    ResponseVariablesGp,
    ResultGp,
    ResultInfoProcedure,
    ResultStatus,
    TaskGp,
} from 'src/app/models';
import { GP_API, TOKENGB } from 'src/app/utility';

import { StorageService } from '../storage/storage.service';

export const UPGP: string = '/api/public/proceduremanager/procedure';
export const PV: string = '/api/public/proceduremanager/procedure/variable'; // -> OK
export const PT: string = '/api/public/proceduremanager/task'; //Attività -> OK
export const PPS: string = '/api/public/proceduremanager/procedure/submissions'; //Attività -> OK
export const PF: string = '/api/public/proceduremanager/procedure/document'; //doc  -> OK
export const DF: string = '/api/public/proceduremanager/procedure/document/download'; //download -> OK
export const PS: string = '/api/public/proceduremanager/procedure/statuses';
export const PFDL: string = '/api/public/proceduremanager/procedure/document/list';  // per la 

//http://156.54.45.165/gestionepratiche/api/public/proceduremanager/procedure/417

@Injectable({
	providedIn: 'root',
})
export class GestorePraticheService {
	constructor(private httpClient: HttpClient, private storageService: StorageService) {}

	generateToken() {
		this.storageService.storeTokenGP(TOKENGB);
	}

	getUserProcedures(filterProcedureGp: FilterProcedureGp): Observable<ResultGp<InfoUserProcedure>> {
		let requesterIdentifier: string = `?requesterIdentifier=${filterProcedureGp.identifier}`;
		let page: string = '&page=' + filterProcedureGp.page;
		let size: string = '&size=' + filterProcedureGp.size;
		let status: string = '&status=' + filterProcedureGp?.status;
		let name: string = '&name=' + filterProcedureGp?.name;
		let protocolNumber: string = '&protocolNumber=' + filterProcedureGp?.protocolNumber;
		let parametri: string = requesterIdentifier + page + size;
		parametri = filterProcedureGp.status ? parametri + status : parametri + '';
		parametri = filterProcedureGp.name ? parametri + name : parametri + '';
		parametri = filterProcedureGp.protocolNumber ? parametri + protocolNumber : parametri + '';
		return this.httpClient.get<ResultGp<InfoUserProcedure>>(`${GP_API}${UPGP}${parametri}`);
	}

	getProcuderesVariables(idProcedure: number): Observable<ResponseVariablesGp> {
		let parametri: string = '/' + idProcedure;
		return this.httpClient.get<ResponseVariablesGp>(`${GP_API}${PV}${parametri}`);
	}

	//TEST
	downloadDocumentGp(request: DownloadFileRqGp): Observable<Blob> {
		return this.httpClient.post(`${GP_API}${DF}`, request, {
			responseType: 'blob',
		});
	}

	getDocumentByProcedure(idProcedure: number): Observable<ResultGp<DocumentGp>> {
		let procedureInstanceId: string = '?procedureInstanceId=' + idProcedure;
		let page: string = '&page=' + '0';
		let size: string = '&size=' + '100';
		let parametri: string = procedureInstanceId + page + size;
		return this.httpClient.get<ResultGp<DocumentGp>>(`${GP_API}${PF}${parametri}`);
	}
	//TEST
	getTaskByUser(identifier: string, idProcedure: number): Observable<ResultGp<TaskGp>> {
		let requesterIdentifier: string = '?requesterIdentifier=' + identifier;
		let page: string = '&page=' + '0';
		let size: string = '&size=' + '100';
		let procedureInstanceId: string = '&procedureInstanceId=' + +idProcedure;
		let parametri: string = requesterIdentifier + page + size + procedureInstanceId;
		return this.httpClient.get<ResultGp<TaskGp>>(`${GP_API}${PPS}${parametri}`);
	}

	getProcedureById(idProcedure: number): Observable<ResultInfoProcedure> {
		let parametri: string = '/' + idProcedure;
		return this.httpClient.get<ResultInfoProcedure>(`${GP_API}${UPGP}${parametri}`);
		// return this.httpClient.get<ResultInfoProcedure>('assets/mock/procedure_info.json');
	}

	getStatuses() {
		return this.httpClient.get<ResultStatus>(`${GP_API}${PS}`);
	}
}
