import { Injectable, NgZone } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { ModalController } from '@ionic/angular';
import { InfoUserProcedure } from 'src/app/models';
import { DettaglioProceduraPage } from 'src/app/pages/gestore-pratiche/dettaglio-procedura/dettaglio-procedura.page';
import { DettaglioNewsPage } from 'src/app/pages/news-eventi/dettaglio-news/dettaglio-news.page';
import { DettaglioPrenotazionePage } from 'src/app/pages/prenotazione/dettaglio-prenotazione/dettaglio-prenotazione.page';
import { ComunicazioneService, GestorePraticheService, GlobalService } from 'src/app/services';
import { DURATIONTOAST, GP_STATUS, MESSAGES } from 'src/app/utility/global-constant';

@Injectable({
	providedIn: 'root',
})
export class ManagePushService {
	constructor(
		private globalService: GlobalService,
		private comunicazioneService: ComunicazioneService,
		private router: Router,
		private modalController: ModalController,
		private activatedRoute: ActivatedRoute,
		private gestorePraticheService: GestorePraticheService,
		private ngZone: NgZone
	) {}

	getComunicazione(idComunicazione: number) {
		this.comunicazioneService.getComunicazione({ id: idComunicazione }).subscribe(
			(result) => {
				if (result.success) {
					if (result.data) {
						let params = this.globalService.encodeParams(result.data);
						this.ngZone.run(() => {
							this.router.navigate(['/tabs/comunicazione', params], { relativeTo: this.activatedRoute });
						});
					}
				}
			},
			(error) => {
				console.log('getComunicazione-error', error);
			}
		);
	}

	goToDettaglioPrenotazione(idPrenotazione: number) {
		this.detailPrenotazioneInModal(idPrenotazione);
	}

	async detailPrenotazioneInModal(idPrenotazione: number) {
		const modal = await this.modalController.create({
			component: DettaglioPrenotazionePage,
			componentProps: {
				idPrenotazione: idPrenotazione,
			},
			showBackdrop: true,
			keyboardClose: false,
			//IONIC 6
			/* initialBreakpoint: 0.8, */
			/* breakpoints: [0, 0.5, 1], */
		});
		return await modal.present();
	}

	goToDetailNews(idNewsEventi: string) {
		this.detailNewsInModal(idNewsEventi);
	}

	async detailNewsInModal(idNewsEventi: string) {
		const modal = await this.modalController.create({
			component: DettaglioNewsPage,
			componentProps: {
				idNe: idNewsEventi,
			},
			showBackdrop: true,
			keyboardClose: false,
			//IONIC 6
			/* initialBreakpoint: 0.8,
			breakpoints: [0, 0.5, 1], */
		});
		return await modal.present();
	}

	goToDetailProcedure(idProcedure: string) {
		this.getProcedureById(idProcedure);
	}

	async detailProcedureInModal(procedure: InfoUserProcedure) {
		const modal = await this.modalController.create({
			component: DettaglioProceduraPage,
			componentProps: {
				procedure: procedure,
			},
			showBackdrop: true,
			keyboardClose: false,
			//IONIC 6
			/* initialBreakpoint: 0.8, */
			/* breakpoints: [0, 0.5, 1], */
		});
		return await modal.present();
	}

	getProcedureById(idProcedure: string) {
		this.gestorePraticheService.getProcedureById(Number(idProcedure)).subscribe(
			(result) => {
				if (result.status === GP_STATUS.status_ok) {
					if (result.item) {
						this.detailProcedureInModal(result.item);
					} else {
						this.globalService.presentToast(MESSAGES.errorDetailProcedure, DURATIONTOAST.default);
					}
				} else {
					this.globalService.presentToast(MESSAGES.errorDetailProcedure, DURATIONTOAST.default);
				}
			},
			() => {
				this.globalService.presentToast(MESSAGES.errorDetailProcedure, DURATIONTOAST.default);
			}
		);
	}
}
