import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { CategoriaPoi, MarkerRequest, PlaceAddress, PlacePortale, PlaceSearchRequest, PlaceTypePortale, Poi } from 'src/app/models';
import { IMAGE, IMAGE_EXTENSION, MACS_API, OPEN_STREET_MAPS } from 'src/app/utility';

import { ResultDTO } from '../../models/resultDTO';
import { OPEN_STREET_MAP_API, PORTALE_ISTITUZIONALE_API } from 'src/app/utility/service.config';
import { map } from 'rxjs/operators';

const GPC = '/configure/1.0/getPoiCategories';
const SOSM = '/search?';
const PS = '/api/mobile/luogo/list';
const PST = '/api/mobile/luogo/type';

@Injectable({
	providedIn: 'root',
})
export class PoiService {
	constructor(private httpClient: HttpClient) { }

	getCategoriePoi(): Observable<ResultDTO<Array<CategoriaPoi>>> {
		return this.httpClient.get<ResultDTO<Array<CategoriaPoi>>>(`${MACS_API}${GPC}`);
	}

	getLocalCategoriePoi(): Observable<Array<CategoriaPoi>> {
		return this.httpClient.get<Array<CategoriaPoi>>('assets/mock/categorie-poi.json');
	}

	getMarkers(markerRequest: MarkerRequest): Observable<Poi[]> {
		let params: string = OPEN_STREET_MAPS.params_search.replace('$1', OPEN_STREET_MAPS.limit_poi.toString()).replace('$2', markerRequest.category);
		params = markerRequest.boxView ? params.replace('$4', markerRequest.boxView) : params.replace(OPEN_STREET_MAPS.viewboxParams, '');
		params = OPEN_STREET_MAPS.enableCity ? params.replace('$3', OPEN_STREET_MAPS.city) : params.replace(OPEN_STREET_MAPS.cityParams, '');
		return this.httpClient.get<Poi[]>(`${OPEN_STREET_MAP_API}${SOSM}${params}`);
	}

	getPlacesFiltered(placeSearchRequest: PlaceSearchRequest): Observable<ResultDTO<PlacePortale[]>> {
		let tipo_luogo = '?tipo_luogo=' + placeSearchRequest.tipo_luogo;
		let titolo = '&titolo=' + placeSearchRequest.titolo;
		let page = '&page=' + placeSearchRequest.page;
		let perPage = '&perPage=' + placeSearchRequest.perPage;
		let parametri = `${tipo_luogo}${titolo}${page}${perPage}`;

		return this.httpClient.get<ResultDTO<PlacePortale[]>>(`${PORTALE_ISTITUZIONALE_API}${PS}${parametri}`)
			.pipe(
				map((response) => {
					response.data = this.remappingData(response.data);
					return response;
				})
			);
	}

	getCoordsByAddress(placeAddress: PlaceAddress): Observable<PlaceAddress[]> {
		let address: string = `${placeAddress.field_indirizzo}${' '}${placeAddress.field_cap}${' '}${placeAddress.field_circoscrizione}${' '}${placeAddress.filed_stato}`;
		let params: string = `${OPEN_STREET_MAPS.linkObtainCoord}`.replace('$1', address);
		return this.httpClient.get<PlaceAddress[]>(`${OPEN_STREET_MAP_API}${SOSM}${params}`);
	}

	getPlacesType(): Observable<ResultDTO<PlaceTypePortale[]>> {
		return this.httpClient.get<ResultDTO<PlaceTypePortale[]>>(`${PORTALE_ISTITUZIONALE_API}${PST} `);
	}

	remappingData(placesPortale: PlacePortale[]): PlacePortale[] {
		placesPortale.map((place) => {
			place.field_image = this.checkSetPathImage(place?.field_image, place?.base_url);
			place.listaImage = this.getListaImage(place?.field_galleria_di_immagini, place?.base_url);
			//	place.body = '';
		})
		return placesPortale;
	}


	checkSetPathImage(linkImage: string, base_url: string): string {
		if (linkImage && linkImage !== '') {
			if (!linkImage.includes('http')) {
				if (base_url && base_url !== '') {
					linkImage = base_url.trim() + '/' + linkImage;
				} else {
					linkImage = IMAGE.neLocandinaTemp;
				}
			}
		} else {
			linkImage = IMAGE.neLocandinaTemp;
		}
		return linkImage;
	}

	getListaImage(listaImageS: string[], base_url: string): Array<string> {
		let listaImage: Array<string> = [];
		if (listaImageS && listaImageS.length > 0) {
			listaImageS.forEach((relativePathImage) => {
				if (relativePathImage && relativePathImage !== '') {
					const isValidExt = IMAGE_EXTENSION.some((ie) => {
						const ext = relativePathImage.slice(relativePathImage.length - ie.length);
						return ext.includes(ie);
					});
					if (isValidExt) {
						if (!relativePathImage.includes('http')) {
							if (base_url && base_url !== '') {
								listaImage.push(base_url.trim() + '/' + relativePathImage.trim());
							}
						} else {
							listaImage.push(relativePathImage.trim());
						}
					}
				}
			});
		}
		return listaImage;
	}


}
