import { Injectable } from '@angular/core';
import { Storage } from '@ionic/storage-angular';

import { DettaglioUtente, InfoStepTwo, PlacePortale, ProcedureStatus, TokenResponseIdp, UserInfo } from '../../models';

const SPID_CODE: string = 'spid_code';
const CF: string = 'codice_fiscale';
const DU: string = 'dettaglio_utenza';
const TUTORIAL: string = 'saw_tutorial';
const CONNECTION: string = 'available_connection';
const TOKENGP: string = 'token_getsione_pratiche';
const SPID_UI: string = 'user_spid_info';
const TOKENIDP: string = 'token_idp';
const TOKENAM: string = 'token_api_manager';
const INFOST: string = 'info_step2_two';
const PROCEDURE_STATUS: string = 'procedure_status';
const PLACE_TO_SHOW: string = 'place_to_show';

@Injectable({
	providedIn: 'root',
})
export class StorageService {
	private _storage: Storage | null = null;

	constructor(public storage: Storage) {
		this.init();
	}

	async init() {
		// If using, define drivers here: await this.storage.defineDriver(/*...*/);
		//RIVEDERE
		const storage = await this.storage.create();
		this._storage = storage;
	}

	storeUserInfo(userInfo: UserInfo): Promise<UserInfo> {
		return this.storage.set(SPID_UI, userInfo).then((userInfo) => {
			return userInfo;
		});
	}

	storeSpidCode(spid_code: string): Promise<string> {
		return this.storage.set(SPID_CODE, spid_code).then((spid_code) => {
			return spid_code;
		});
	}

	storeCF(cf: string): Promise<string> {
		return this.storage.set(CF, cf).then((cf) => {
			return cf;
		});
	}

	storeDettaglioUtenza(dettaglioUtenza: DettaglioUtente): Promise<DettaglioUtente> {
		return this.storage.set(DU, dettaglioUtenza).then((dettaglioUtenza) => {
			return dettaglioUtenza;
		});
	}

	storeSawTutorial(saw: boolean): Promise<boolean> {
		return this.storage.set(TUTORIAL, saw).then((value) => {
			return value;
		});
	}

	storeConnectionStatus(status: boolean): Promise<boolean> {
		return this.storage.set(CONNECTION, status).then((status) => {
			return status;
		});
	}

	storeTokenGP(token: string): Promise<boolean> {
		return this.storage.set(TOKENGP, token).then((token) => {
			return token;
		});
	}

	storeTokenIdp(tokenResponseIdp: TokenResponseIdp): Promise<boolean> {
		return this.storage.set(TOKENIDP, tokenResponseIdp).then((token) => {
			return token;
		});
	}

	storeTokenAm(token: string): Promise<boolean> {
		return this.storage.set(TOKENAM, token).then((token) => {
			return token;
		});
	}

	storeInfoStep2(infoStep2: InfoStepTwo): Promise<boolean> {
		return this.storage.set(INFOST, infoStep2).then((infoStep2) => {
			return infoStep2;
		});
	}

	storeProcedureStatus(procedureStatus: ProcedureStatus[]): Promise<boolean> {
		return this.storage.set(PROCEDURE_STATUS, procedureStatus).then((procedureStatus) => {
			return procedureStatus;
		});
	}

	storePlaceToShow(place: PlacePortale): Promise<boolean> {
		return this.storage.set(PLACE_TO_SHOW, place).then((place) => {
			return place;
		});
	}

	getUserInfo(): Promise<UserInfo> {
		return this.storage.get(SPID_UI).then((userInfo) => {
			return userInfo?.identifier ? userInfo : { ...userInfo, identifier: 'RSSPLA80A01H501T' };
		});
	}

	getSpidCode(): Promise<string> {
		return this.storage.get(SPID_CODE).then((spid_code) => {
			return spid_code;
		});
	}

	getDettaglioUtenza(): Promise<DettaglioUtente> {
		return this.storage.get(DU).then((dettaglioUtenza) => {
			return dettaglioUtenza;
		});
	}

	getCF(): Promise<string> {
		return this.storage.get(CF).then((cf) => {
			return cf;
		});
	}

	getSawTutorial(): Promise<boolean> {
		return this.storage.get(TUTORIAL).then((value) => {
			return value;
		});
	}

	getConnectionStatus(): Promise<boolean> {
		return this.storage.get(CONNECTION).then((status) => {
			return status;
		});
	}

	getTokenGP(): Promise<string> {
		return this.storage.get(TOKENGP).then((token) => {
			return token;
		});
	}

	getTokenIdp(): Promise<TokenResponseIdp> {
		return this.storage.get(TOKENIDP).then((tokenResponseIdp) => {
			return tokenResponseIdp;
		});
	}

	getTokenAm(): Promise<string> {
		return this.storage.get(TOKENAM).then((token) => {
			return token;
		});
	}

	getInfoStep2(): Promise<InfoStepTwo> {
		return this.storage.get(INFOST).then((infoStep2) => {
			return infoStep2;
		});
	}

	getProcedureStatus(): Promise<ProcedureStatus[]> {
		return this.storage.get(PROCEDURE_STATUS).then((procedureStatus) => {
			return procedureStatus;
		});
	}

	getPlaceToShow(): Promise<PlacePortale> {
		return this.storage.get(PLACE_TO_SHOW).then((place) => {
			return place;
		});
	}

	removeUserInfo(): Promise<boolean> {
		return this.storage.remove(SPID_UI);
	}

	removeSpidCode(): Promise<boolean> {
		return this.storage.remove(SPID_CODE);
	}

	removeCF(): Promise<boolean> {
		return this.storage.remove(CF);
	}

	removeDettaglioUtenza(): Promise<boolean> {
		return this.storage.remove(DU);
	}

	removeSawTutorial(): Promise<boolean> {
		return this.storage.remove(TUTORIAL);
	}

	removeConnectionStatus(): Promise<boolean> {
		return this.storage.remove(CONNECTION);
	}

	removeTokenGP(): Promise<boolean> {
		return this.storage.remove(TOKENGP);
	}

	removeTokenIdp(): Promise<TokenResponseIdp> {
		return this.storage.remove(TOKENIDP);
	}

	removeTokenAm(): Promise<boolean> {
		return this.storage.remove(TOKENAM);
	}

	removeInfoStep2(): Promise<InfoStepTwo> {
		return this.storage.remove(INFOST);
	}
	
	removeProcedureStatus(): Promise<ProcedureStatus[]> {
		return this.storage.remove(PROCEDURE_STATUS);
	}

	removePlaceToShow(): Promise<PlacePortale> {
		return this.storage.remove(PLACE_TO_SHOW);
	}

	removeAll() {
		this.removeDettaglioUtenza();
		//this.removeSawTutorial();
		this.removeTokenGP();
		this.removeTokenIdp();
		this.removeTokenAm();
		this.removeUserInfo();
		this.removeSpidCode();
		this.removeInfoStep2();
		this.removeProcedureStatus();
		this.removePlaceToShow();
	}
}
