import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { DettaglioUtente, ResultDTO, TipologiaNotifica, UpdateTipologieNotificheRequest } from 'src/app/models';
import { MACS_API, MACS_APPID } from 'src/app/utility';

const GET_DU: string = '/service/1.0/getDettaglioUtenza';
const GET_TIPOLOGIE: string = '/configure/1.0/getNotifichePerUtente';
const UPDATE_TIPOLOGIE: string = '/configure/1.0/updateNotifichePerUtente';

@Injectable({
  providedIn: 'root'
})
export class UtenzaService {

  constructor(private httpClient: HttpClient) { }

  getDettaglioUtenza(id_user: string): Observable<ResultDTO<DettaglioUtente>> {
    const httpOptions = {
      headers: new HttpHeaders({
        'Content-Type': 'application/json',
        'ApplicationId': MACS_APPID
      })
    }
    let parametri = '?t_id_user=' + id_user;
    return this.httpClient.get<ResultDTO<DettaglioUtente>>(`${MACS_API}${GET_DU}${parametri}`, httpOptions);
  }

  getNotifichePerUtente(spidCode: string): Observable<ResultDTO<Array<TipologiaNotifica>>> {
    let parametri = '?t_spid_code=' + spidCode;
    return this.httpClient.get<ResultDTO<Array<TipologiaNotifica>>>(`${MACS_API}${GET_TIPOLOGIE}${parametri}`)
  }

  updateNotifichePerUtente(updateTipologieNotificheRequest: UpdateTipologieNotificheRequest): Observable<ResultDTO<String>> {
    return this.httpClient.post<ResultDTO<String>>(`${MACS_API}${UPDATE_TIPOLOGIE}`, updateTipologieNotificheRequest);
  }
}
