package it.itsvil.macs.backend.client;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.Invocation;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.gson.Gson;

import it.itsvil.macs.backend.dto.NotificatorePushBrodCastRequestDTO;
import it.itsvil.macs.backend.dto.NotificatorePushRequestDTO;
import it.itsvil.macs.backend.dto.NotificatorePushResponseDTO;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service(value = "NotificatoreClientImpl")
public class NotificatoreClientImpl implements NotificatoreClient {
	
	@Autowired
	private Environment env;
	
	@Override
	public NotificatorePushResponseDTO sendPushNotification(NotificatorePushRequestDTO request) {
		
		NotificatorePushResponseDTO response=null;
		
		log.info("START");
		
		try {
			ObjectMapper mapper = new ObjectMapper();
			String jsonString = mapper.writeValueAsString(request);
			log.info("REQUEST: {}", jsonString);
		} catch (JsonProcessingException e) {
			log.error("ERRORE: {}", e.getMessage());
		}
		
		try {

			String output = null;

			response = new NotificatorePushResponseDTO();

			String url = env.getProperty("macs.notificatore.sendPushNotification.url");
			
			String applicationId=env.getProperty("macs.notificatore.applicationId");
			
			if (null==applicationId || StringUtils.isBlank(applicationId))
				throw new Exception("macs.notificatore.applicationId = "+applicationId);
			
			if (null==url || StringUtils.isBlank(url))
				throw new Exception("macs.notificatore.sendPushNotification.url = "+url);

			Client client = ClientBuilder.newClient();

			log.info("URL: {}", url);

			WebTarget webTarget = client.target(url);

			Invocation.Builder invocationBuilder = webTarget.request().header("ApplicationId", applicationId).header("cache-control", "no-cache");
			Response resp = invocationBuilder.post(Entity.entity(request, MediaType.APPLICATION_JSON_VALUE));

			log.info("SERVIZIO INVOCATO");

			if(resp.getStatus() == 401){
				log.error("ERRORE: 401 unauthorized");
				throw new Exception("NotificatoreClientImpl.sendPushNotification.Client internal unauthorized");
			}

			if (resp.getStatus() < 200 || resp.getStatus() >= 300) {
				log.error("ERRORE: Failed HTTP error code");
				throw new Exception("NotificatoreClientImpl.sendPushNotification. Failed HTTP error code . "
						+ resp.getStatus());
			}
			
			output = resp.readEntity(String.class);
			
			try {
				log.info("RESPONSE: {}", output);
			}catch (Exception e) {
				log.error("ERRORE: {}", e.getMessage());
			}
			
			Gson gson = new Gson(); 
			response = gson.fromJson(output ,NotificatorePushResponseDTO.class );

		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}

		log.info("END");
		return response;
	}

	@Override
	public NotificatorePushResponseDTO sendPushNotification(NotificatorePushBrodCastRequestDTO request) {
		NotificatorePushResponseDTO response=null;
		
		log.info("START");
		
		try {
			ObjectMapper mapper = new ObjectMapper();
			String jsonString = mapper.writeValueAsString(request);
			log.info("REQUEST: {}", jsonString);
		} catch (JsonProcessingException e) {
			log.error("ERRORE: {}", e.getMessage());
		}
		
		try {

			String output = null;

			response = new NotificatorePushResponseDTO();

			String url = env.getProperty("macs.notificatore.sendBroadcastPushNotification.url");
			
			String applicationId=env.getProperty("macs.notificatore.applicationId");
			
			if (null==applicationId || StringUtils.isBlank(applicationId))
				throw new Exception("macs.notificatore.applicationId = "+applicationId);
			
			if (null==url || StringUtils.isBlank(url))
				throw new Exception("macs.notificatore.sendBroadcastPushNotification.url = "+url);

			Client client = ClientBuilder.newClient();

			log.info("URL: {}", url);

			WebTarget webTarget = client.target(url);

			Invocation.Builder invocationBuilder = webTarget.request().header("ApplicationId", applicationId).header("cache-control", "no-cache");
			Response resp = invocationBuilder.post(Entity.entity(request, MediaType.APPLICATION_JSON_VALUE));

			log.info("SERVIZIO INVOCATO");

			if(resp.getStatus() == 401){
				log.error("ERRORE: 401 unauthorized");
				throw new Exception("NotificatoreClientImpl.sendPushNotification.Brodacast.Client internal unauthorized");
			}

			if (resp.getStatus() < 200 || resp.getStatus() >= 300) {
				log.error("ERRORE: Failed HTTP error code");
				throw new Exception("NotificatoreClientImpl.sendPushNotification.Brodacast Failed HTTP error code . "
						+ resp.getStatus());
			}
			
			output= resp.readEntity(String.class);
			
			try {
				log.info("RESPONSE: {}", output);
			}catch (Exception e) {
				log.error("ERRORE: {}", e.getMessage());
			}
			
			Gson gson = new Gson(); 
			response = gson.fromJson(output ,NotificatorePushResponseDTO.class );

		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}

		log.info("END");
		return response;
	}

}
