package it.itsvil.macs.backend.client;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.Invocation;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.MultivaluedHashMap;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.Response;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.gson.Gson;

import it.itsvil.macs.backend.dto.ProfilatoreDettaglioUtenteRequestDTO;
import it.itsvil.macs.backend.dto.ProfilatoreDettaglioUtenteResponseDTO;
import it.itsvil.macs.backend.dto.ProfilatoreInsertUtenzaByRuoloRequestDTO;
import it.itsvil.macs.backend.dto.ProfilatoreInsertUtenzaByRuoloResponseDTO;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service(value = "ProfilatoreClientImpl")
public class ProfilatoreClientImpl implements ProfilatoreClient {
	
	@Autowired
	private Environment env;
	
	@Value(value = "${macs.profilatore.applicationId}")
	private String applicationId;
	
	@Value(value = "${macs.profilatore.insertUtenzaByRuolo.url}")
	private String insertUtenzaByRuoloUrl;

	@Override
	public ProfilatoreDettaglioUtenteResponseDTO getDettaglioUtenza(ProfilatoreDettaglioUtenteRequestDTO request) {
		
		
		ProfilatoreDettaglioUtenteResponseDTO response = null;
		String output;
		
		log.info("START");
		
		try {
			ObjectMapper mapper = new ObjectMapper();
			String jsonString = mapper.writeValueAsString(request);
			log.info("REQUEST: {}", jsonString);
		} catch (JsonProcessingException e) {
			log.error("ERRORE: {}", e.getMessage());
		}
		
		try {
			
			String applicationId = env.getProperty("macs.profilatore.applicationId");
			String url = env.getProperty("macs.profilatore.getDettaglioUtenza.url");
			
			if (null==applicationId || StringUtils.isBlank(applicationId))
				throw new Exception("macs.profilatore.applicationId = "+applicationId);
			
			if (null==url || StringUtils.isBlank(url))
				throw new Exception("macs.profilatore.getDettaglioUtenza.url = "+url);

			
			Client client = ClientBuilder.newClient();
			
			log.info("URL: {}", url);
			
			WebTarget webTarget = client.target(url).queryParam("t_id_user", request.getT_id_user());
			Invocation.Builder invocationBuilder = webTarget.request().header("ApplicationID", applicationId).header("cache-control", "no-cache");
			Response responseClient = invocationBuilder.get();
			
			log.info("SERVIZIO INVOCATO");
			
			if(responseClient.getStatus() == 401){
				log.error("ERRORE: 401 unauthorized");
				throw new Exception("ProfilatoreClientImpl.getDettaglioUtenza.Client internal unauthorized");
			}

			if (responseClient.getStatus() < 200 || responseClient.getStatus() >= 300) {
				log.error("ERRORE: Failed HTTP error code");
				throw new Exception("ProfilatoreClientImpl.getDettaglioUtenza. Failed HTTP error code . "
						+ responseClient.getStatus());
			}
			
			output = responseClient.readEntity(String.class);
			
			try {
				log.info("RESPONSE: {}", output);
			}catch (Exception e) {
				log.error("ERRORE: {}",e.getMessage());
			}
			
			Gson gson = new Gson(); 
			response = gson.fromJson(output, ProfilatoreDettaglioUtenteResponseDTO.class );
			
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}

		log.info("END");
		return response;
	}

	@Override
	public ProfilatoreInsertUtenzaByRuoloResponseDTO insertUtenzaByRuolo(
			ProfilatoreInsertUtenzaByRuoloRequestDTO request) {
		
		log.info("START");
		log.info("ApplicationId: {}", applicationId);
		log.info("insertUtenzaByRuolo URL: {}", insertUtenzaByRuoloUrl);
		
		try {
			String jsonStringRequest = new ObjectMapper().writeValueAsString(request);
			log.info("REQUEST: {}", jsonStringRequest);
			
			Client client = ClientBuilder.newClient();
			
			WebTarget webTarget = client.target(insertUtenzaByRuoloUrl);
			
			MultivaluedMap<String, Object> headers = new MultivaluedHashMap<String, Object>();
			headers.add("cache-control", "no-cache");
			headers.add("ApplicationID", applicationId);
			
			Invocation.Builder invocationBuilder = webTarget
					.request()
					.headers(headers);
			
			Response responseFromClient = invocationBuilder.put(Entity.entity(request, MediaType.APPLICATION_JSON_VALUE));
			
			log.info("SERVIZIO INVOCATO");
			
			String output = responseFromClient.readEntity(String.class);
			log.info("RESPONSE STRING FROM CLIENT: {}", output);
			
			if(responseFromClient.getStatus() == 401){
				log.error("ERRORE: 401 unauthorized");
				throw new Exception("ProfilatoreClientImpl.insertUtenzaByRuolo. internal unauthorized");
			}

			if (responseFromClient.getStatus() < 200 || responseFromClient.getStatus() >= 300) {
				log.error("ERRORE: Failed HTTP error code");
				throw new Exception("ProfilatoreClientImpl.insertUtenzaByRuolo. Failed HTTP error code: "
						+ responseFromClient.getStatus());
			}
			
			ProfilatoreInsertUtenzaByRuoloResponseDTO response =
					new Gson().fromJson(output, ProfilatoreInsertUtenzaByRuoloResponseDTO.class);
			
			log.info("END");
			return response;
		} catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}
	}

}
