package it.itsvil.macs.backend.service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import it.itsvil.macs.backend.client.NotificatoreClient;
import it.itsvil.macs.backend.dao.ComunicazioneDao;
import it.itsvil.macs.backend.dao.DispositivoDao;
import it.itsvil.macs.backend.dto.ComunicazioneDTO;
import it.itsvil.macs.backend.dto.ComunicazioneRequestDTO;
import it.itsvil.macs.backend.dto.NotificationInfoDTO;
import it.itsvil.macs.backend.dto.NotificatorePushBrodCastRequestDTO;
import it.itsvil.macs.backend.dto.NotificatorePushResponseDTO;
import it.itsvil.macs.backend.dto.NotificatoreTokenDTO;
import it.itsvil.macs.backend.util.MacsEnum;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service(value = "ComunicazioneServiceImpl")
public class ComunicazioneServiceImpl implements ComunicazioneService {
	
	@Autowired
	private ComunicazioneDao dao;
	
	@Autowired
	private DispositivoDao dispositivoDao;
	
	@Autowired
	private NotificatoreClient client;
	
	@Autowired
	private Environment env;

	@Override
	public ComunicazioneDTO getComunicazioneById(Long n_id) {

		ComunicazioneDTO result = null;
		
		log.info("START");
		
		result = dao.getComunicazioneById(n_id);
		
		log.info("END");
		
		return result;
	}

	@Override
	public Long insertComunicazione(ComunicazioneRequestDTO request) {
		Long result = null;
		
		log.info("[ComunicazioneServiceImpl.insertComunicazione]START");
		
		ComunicazioneDTO insertValue = new ComunicazioneDTO();
		
		insertValue.setD_data_ins(LocalDateTime.now());
		insertValue.setT_allegato(request.getT_allegato());
		insertValue.setT_descrizione_breve(request.getT_descrizione_breve());
		insertValue.setT_testo_completo(request.getT_testo_completo());
		insertValue.setT_titolo(request.getT_titolo());
		
		result = dao.insertComunicazione(insertValue);
		
		String code= env.getProperty("macs.notificatore.comunicazione.code");
		if (null==code || StringUtils.isBlank(code)) {
			log.error("ERRORE: codice per la push mancante dal property");
			return null;
		}
		
		List<NotificationInfoDTO> listOfDevice = dispositivoDao.listOfAllLoggedAvaibleDevice(code);
		
		if (null==listOfDevice || listOfDevice.size()<=0) {
			log.info("END: lista di device vuota");
			return (long) 0 ;
		}
		
		NotificatorePushBrodCastRequestDTO clientRequest = new  NotificatorePushBrodCastRequestDTO();
		List<NotificatoreTokenDTO> listOfToken= new ArrayList<NotificatoreTokenDTO>();
		
		for (NotificationInfoDTO n:listOfDevice) {
			NotificatoreTokenDTO token= new NotificatoreTokenDTO();
			token.setSistema_operativo(n.getT_piattaforma());
			token.setToken(n.getT_token_firebase());
			listOfToken.add(token);
		}
		clientRequest.setListOfToken(listOfToken);
		clientRequest.setPushTitle(request.getT_titolo());
		clientRequest.setPushMessage(request.getT_descrizione_breve());
		
		Map<String,String> parametriAggiuntivi = new HashMap<String,String>();
		parametriAggiuntivi.put(MacsEnum.CODICE.getCodice(), code);
		parametriAggiuntivi.put(MacsEnum.VALORE.getCodice(), result.toString());
		clientRequest.setParametriAggiuntivi(parametriAggiuntivi);
		
		NotificatorePushResponseDTO responseClient = client.sendPushNotification(clientRequest);
		if (null == responseClient) {
			log.error("ERRORE: impossibile contattare il notificatore");
			return null;
		}
		
		log.info("END");
		return result;
	}
	
}
