package it.itsvil.macs.backend.service;

import java.io.File;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import com.google.gson.Gson;
import it.itsvil.macs.backend.dto.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import it.itsvil.macs.backend.client.PrenotazioneClient;
import it.itsvil.macs.backend.client.ProfilatoreClient;
import it.itsvil.macs.backend.dao.MacsDao;
import it.itsvil.macs.backend.util.MacsEnum;
import it.itsvil.macs.backend.util.McasTipologiaEnum;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service(value = "MacsConfigurazionServiceImpl")
public class MacsServiceImpl implements MacsService {

    @Autowired
    private PrenotazioneClient client;

    @Autowired
    private Environment env;

    @Autowired
    private MacsDao dao;

    @Autowired
    private DispositivoService dispositivoService;

    @Autowired
    private ZoomService zoomService;

    @Autowired
    private ProfilatoreClient profilatoreClient;

    @Value(value = "${macs.prenotazione.test}")
    public Boolean PREN_TEST;


    @Override
    public String keepAlive(int id) {
        if (id == 0)
            dispositivoService.listOfNotification();
        else if (id == 1)
            dispositivoService.remainderToUse();
        return "Sono vivo dal Backend";
    }

    @Override
    public void audit(AuditRequestDTO request) {
        log.debug("[MacsServiceImpl.audit]START");
        String jsonString = null;
        try {
            ObjectMapper mapper = new ObjectMapper();
            if (null != request.getT_slot_occupati())
                jsonString = mapper.writeValueAsString(request.getT_slot_occupati());
        } catch (JsonProcessingException e) {
            log.error("ERRORE: Errore nello trasformazione della lista degli slot occupati -> {}", e.getMessage());
        }

        AuditDTO auditDTO = new AuditDTO();
        auditDTO.setT_nome_ufficio(request.getT_nome_ufficio());
        auditDTO.setN_id_ufficio(request.getN_id_ufficio());
        auditDTO.setT_nome_risorsa(request.getT_nome_risorsa());
        auditDTO.setN_id_risorsa(request.getN_id_risorsa());
        auditDTO.setD_data_inizio_appuntamento(request.getD_data_inizio_appuntamento());
        auditDTO.setD_data_fine_appuntamento(request.getD_data_fine_appuntamento());
        auditDTO.setD_data_prenotazione(request.getD_data_prenotazione());
        auditDTO.setT_questionario_compilato(request.getT_questionario_compilato());

        if (request.getT_tipologia_prenotazione().equalsIgnoreCase(McasTipologiaEnum.PerTe.getCodice()))
            auditDTO.setT_tipologia_prenotazione(McasTipologiaEnum.PerTe.getDescrizione());
        else if (request.getT_tipologia_prenotazione().equalsIgnoreCase(McasTipologiaEnum.PerTerzi.getCodice()))
            auditDTO.setT_tipologia_prenotazione(McasTipologiaEnum.PerTerzi.getDescrizione());

        auditDTO.setT_slot_occupati(jsonString);
        auditDTO.setN_id_prenotazione(request.getN_id_prenotazione());
        auditDTO.setT_disdetto(MacsEnum.NO.getCodice());
        dao.insertAudit(auditDTO);
        log.debug("END");

    }

    @Override
    public void upDateAudit(int idPrenotazione) {
        log.debug("[MacsServiceImpl.upDateAudit]START");

        AuditDTO auditDTO = new AuditDTO();
        auditDTO.setN_id_prenotazione((long) idPrenotazione);
        auditDTO.setD_data_disdetta(LocalDateTime.now());
        auditDTO.setT_disdetto(MacsEnum.SI.getCodice());
        dao.upDateAudit(auditDTO);

        log.debug("END");
    }


    @Override
    public ProviderListResponseDTO providerList() {

        List<ProviderDTO> list = new ArrayList<ProviderDTO>();

        String basePath = env.getProperty("macs.providerListImg.url");
        String ipServer = env.getProperty("macs.server.ip");
        String baseDirectory = env.getProperty("macs.baseDirectory");
        String fullpath = System.getProperty("catalina.base") + basePath;

        File directory = new File(fullpath);
        File[] fileNames = directory.listFiles();
        for (File file : fileNames) {
            ProviderDTO provider = new ProviderDTO();

            provider.setId("adsasdd");
            provider.setNome(file.getName());

            String pathFinale = "http://" + ipServer + ":80/" + baseDirectory + "/" + file.getName();
            provider.setImage(pathFinale);
            list.add(provider);

        }
        ProviderListResponseDTO response = new ProviderListResponseDTO();
        response.setProviderList(list);

        return response;
    }

    @Override
    public ListaRisorseMobileAppResponse listaUffici(CoreRequestDTO request) {
        log.info("START");

        ListaRisorseMobileAppResponse risultato = new ListaRisorseMobileAppResponse();
        List<RisorseDTO> risorse = new ArrayList<RisorseDTO>();

        ListaRisorseResponseDTO clientResponse = client.listaRisorse(request);

        if (null == clientResponse) {
            log.debug("ERRORE: RISPOSTA NULL DAL CLIENT");
            return null;
        }

        if (!clientResponse.isSuccess()) {
            log.debug("ERRORE: some problem appened from client: {}", clientResponse.getError());
            risultato.setRisorseUfficioDTO(new ArrayList<RisorseDTO>());
            risultato.setSuccess(clientResponse.isSuccess());
            risultato.setError(clientResponse.getError());
            return risultato;
        }
        //Caso in cui non ci sono stati errori nel client
        else {
            //Caso in cui il client restituisce un risultato non valido
            if (null == clientResponse.getContent() || null == clientResponse.getContent().getRisorse()) {
                log.debug("ERRORE: content is null or list of resource null: ");
                risultato.setRisorseUfficioDTO(new ArrayList<RisorseDTO>());
                risultato.setSuccess(clientResponse.isSuccess());
                risultato.setError(clientResponse.getError());
                return risultato;
            }
            //Caso in cui il client restituisce un risultato valido
            else {
                log.info("RESPONSE FROM CLIENT OK");

                List<RisorseDTO> listaRClient = clientResponse.getContent().getRisorse();
                for (RisorseDTO risorsaClient : listaRClient) {
                    if (risorsaClient.getIdParent() == -1) {
                        RisorseMobileDTO r = new RisorseMobileDTO();

                        r.setCompleteRisorsa(risorsaClient, countNumerOfResource(risorsaClient.getId(), listaRClient));

                        risorse.add(r);
                    }

                }
                risultato.setSuccess(clientResponse.isSuccess());
                risultato.setError(clientResponse.getError());
                risultato.setRisorseUfficioDTO(risorse);
            }
        }

        log.info("END");
        return risultato;
    }

    private int countNumerOfResource(int id, List<RisorseDTO> listaRClient) {
        int count = 0;
        for (RisorseDTO c : listaRClient) {
            if (c.getIdParent() == id)
                count++;
        }
        return count;
    }

    @Override
    public ListaRisorseMobileAppResponse listaRisorsa(ListaRisorseMobileAppRequest request) {

        log.info("START");

        ListaRisorseMobileAppResponse risultato = new ListaRisorseMobileAppResponse();
        List<RisorseDTO> risorse = new ArrayList<RisorseDTO>(); //Risorse da restituire

        ListaRisorseResponseDTO clientResponse = client.listaRisorse(request);

        if (null == clientResponse) {
            log.debug("NULL RESPONSE FROM CLIENT");
            return null;
        }

        //Caso in cui il client restiuisce un risultato non valido
        if (!clientResponse.isSuccess() || null == clientResponse.getContent() || null == clientResponse.getContent().getRisorse()) {
            log.debug("some problem apped from client: {}", clientResponse.getError());

            risultato.setRisorseUfficioDTO(new ArrayList<RisorseDTO>());
            risultato.setSuccess(clientResponse.isSuccess());
            risultato.setError(clientResponse.getError());

            return risultato;
        }
        //Caso in cui il client restituisce un risulato valido
        else {
            log.info("RESPONSE FROM CLIENT OK");

            List<RisorseDTO> listaRClient = clientResponse.getContent().getRisorse();
            for (RisorseDTO risorsaClient : listaRClient) {
                if (risorsaClient.getIdParent() == request.getId()) {
                    RisorseMobileDTO r = new RisorseMobileDTO();

                    r.setCompleteRisorsa(risorsaClient, risorsaClient.getIdParent());

                    risorse.add(r);
                }
            }
            risultato.setSuccess(clientResponse.isSuccess());
            risultato.setError(clientResponse.getError());
            risultato.setRisorseUfficioDTO(risorse);
        }

        log.info("END");
        return risultato;
    }

    @Override
    public InfoRisorseResponseDTO infoRisorse(InfoRisorseRequestDTO request) {
        log.info("START");

        InfoRisorseResponseDTO clientResponse = client.infoRisorse(request);

        if (null == clientResponse) {
            log.debug("NULL RESPONSE FROM CLIENT");
            return null;
        }

        if (!clientResponse.isSuccess() || null == clientResponse.getContent()) {
            log.debug("some problem apped from client: {}", clientResponse.getError());
            return clientResponse;
        } else {
            log.info("END");
            return clientResponse;
        }
    }


    @Override
    public PeriodiDisponibiliResponseDTO periodiDisponibili(PeriodiDisponibiliRequestDTO request) {
        log.info("START");

        PeriodiDisponibiliResponseDTO clientResponse = client.periodiDisponibili(request);

        if (null == clientResponse) {
            log.debug("NULL RESPONSE FROM CLIENT");
            return null;
        }

        if (!clientResponse.isSuccess() || null == clientResponse.getContent()) {
            log.debug("some problem apped from client: {}", clientResponse.getError());
            return clientResponse;
        } else {
            log.info("END");
            List<SlotDTO> lista = clientResponse.getContent().getSlots();
            for (SlotDTO s : lista) {
                s.setInizio(s.getInizio().replace("string", request.getDa()));
                s.setFine(s.getFine().replace("string", request.getDa()));
            }
            return clientResponse;
        }
    }

    @Override
    public DownloadDocumentoResponseDTO downloadDocumento(DownloadDocumentoRequestDTO request) {

        log.info("START");

        DownloadDocumentoResponseDTO clientResponse = client.downloadDocumento(request);

        if (null == clientResponse) {
            log.debug("NULL RESPONSE FROM CLIENT");
            return null;
        }

        if (!clientResponse.isSuccess() || null == clientResponse.getContent()) {
            log.debug("some problem apped from client: {}", clientResponse.getError());
            return clientResponse;
        } else {
            log.info("END");
            return clientResponse;
        }
    }

    @Override
    public InitPrenotazioneResponseDTO initPrenotazione(InitPrenotazioneRequestDTO request) {

        log.info("START");

        InitPrenotazioneResponseDTO clientResponse = client.initPrenotazione(request);

        if (null == clientResponse) {
            log.debug("NULL RESPONSE FROM CLIENT");
            return null;
        }

        if (!clientResponse.isSuccess() || null == clientResponse.getContent()) {
            log.debug("some problem apped from client: {}", clientResponse.getError());
            return clientResponse;
        } else {
            log.info("END");
            return clientResponse;
        }
    }

    @Override
    public AnnullaPrenotazioneResponseDTO annullaPrenotazione(AnnullaPrenotazioneRequestDTO request) {

        log.info("START");

        AnnullaPrenotazioneResponseDTO clientResponse = client.annullaPrenotazione(request);

        if (null == clientResponse) {
            log.debug("NULL RESPONSE FROM CLIENT");
            return null;
        }

        if (!clientResponse.isSuccess() || null == clientResponse.getContent()) {
            log.debug("some problem apped from client: {}", clientResponse.getError());
            return clientResponse;
        } else {
            log.info("END");
            return clientResponse;
        }

    }

    @Override
    public ConfermaPrenotazioneResponseDTO confermaPrenotazione(ConfermaPrenotazioneCustomRequestDTO request) {

        log.info("START");

        ConfermaPrenotazioneResponseDTO clientResponse = client.confermaPrenotazione(request.getConfermaPrenotazione());

        if (null == clientResponse) {
            log.debug("NULL RESPONSE FROM CLIENT");
            return null;
        }

        if (!clientResponse.isSuccess() || null == clientResponse.getContent()) {
            log.debug("some problem append from client: {}", clientResponse.getError());
            return clientResponse;
        } else {

            ZoomMeetingObjectDTO meeting = null;

            try {
                zoomService.createMeeting(request.getMeeting());
            } catch (Exception e) {
                e.printStackTrace();
            }

            salvaPrenotazione(request, clientResponse, meeting!=null ? meeting.getId()+"" : null);

            log.info("END");
            return clientResponse;
        }
    }


    private PrenotazioneDTO salvaPrenotazione(ConfermaPrenotazioneCustomRequestDTO request, ConfermaPrenotazioneResponseDTO clientResponse, String t_id_meeting) {
        log.info("START");

        PrenotazioneDTO prenotazioneDTO = new PrenotazioneDTO();

        prenotazioneDTO.setD_data_appuntamento(request.getDataInizioPrenotazione());
        prenotazioneDTO.setN_id_prenotazione((long) clientResponse.getContent().getIdPrenotazione());
        prenotazioneDTO.setN_id_risorsa(request.getIdRisorsa());
        prenotazioneDTO.setN_id_ufficio(request.getIdUfficio());
        prenotazioneDTO.setT_disdetto(MacsEnum.NO.getCodice());
        prenotazioneDTO.setT_nome_risorsa(request.getNomeRisorsa());
        prenotazioneDTO.setT_nome_ufficio(request.getNomeUfficio());
        prenotazioneDTO.setT_cf_fruitore(request.getConfermaPrenotazione().getCfFruitore());
        prenotazioneDTO.setT_id_meeting_zoom(t_id_meeting);

        if (request.getPrenotazionePersonale().equalsIgnoreCase(MacsEnum.SI.getCodice()))
            prenotazioneDTO.setT_prenotazione_personale(MacsEnum.SI.getCodice());
        else if (request.getPrenotazionePersonale().equalsIgnoreCase(MacsEnum.NO.getCodice()))
            prenotazioneDTO.setT_prenotazione_personale(MacsEnum.NO.getCodice());
        else log.warn("CODICE PRENOTAZIONE PERSONALE NON CORRETTO: {}", request.getPrenotazionePersonale());

        prenotazioneDTO.setT_spid_code(request.getSpidCode());
        dao.insertPrenotazione(prenotazioneDTO);

        log.info("END");
        return prenotazioneDTO;
    }



    // t_satisfaction
    // 01 sorriso
    // 02 triste
    // ...
    // null non valorizzato
    public void updateSatisfactionPrenotazione(int n_id_prenotazione, String t_satisfaction) {
        log.info("START");

        PrenotazioneDTO prenotazioneDTO = new PrenotazioneDTO();

        prenotazioneDTO.setN_id_prenotazione((long) n_id_prenotazione);
        prenotazioneDTO.setT_satisfaction(t_satisfaction);

        dao.upDateSadisfactionPrenotazione(prenotazioneDTO);

        log.info("END");
    }



    @Override
    public DisdiciPrenotazioneResponseDTO disdiciPrenotazione(DisdiciPrenotazioneRequestDTO request) {

        log.info("START");

        DisdiciPrenotazioneResponseDTO clientResponse = client.disdiciPrenotazione(request);

        if (null == clientResponse) {
            log.debug("NULL RESPONSE FROM CLIENT");
            return null;
        }

        if (!clientResponse.isSuccess() || null == clientResponse.getContent()) {
            log.debug("some problem apped from client: {}", clientResponse.getError());
            return clientResponse;
        } else {

            // Chiamata a zoom parallela al ruturn del metodo
            new Thread(new Runnable() {
                @Override
                public void run() {
                    PrenotazioneDTO prenotazioneDTO = dao.getPrenotazione((long) request.getIdPrenotazione());
                    try{
                        zoomService.deleteMeeting(prenotazioneDTO.getT_id_meeting_zoom());
                    }catch(Exception e){ e.printStackTrace(); }
                    try{
                        prenotazioneDTO.setT_id_meeting_zoom(null);
                        dao.upDateMeetingPrenotazione(prenotazioneDTO);
                    }catch(Exception e){ e.printStackTrace(); }
                }
            }).start();

            upDateAudit(request.getIdPrenotazione());
            upDatePrenotazioneCorrente(request.getIdPrenotazione());
            log.info("END");
            return clientResponse;
        }
    }

    private void upDatePrenotazioneCorrente(int idPrenotazione) {
        log.info("START");

        PrenotazioneDTO prenotazioneDTO = new PrenotazioneDTO();
        prenotazioneDTO.setN_id_prenotazione((long) idPrenotazione);
        prenotazioneDTO.setT_disdetto(MacsEnum.SI.getCodice());
        dao.UpDateDisdiciPrenotazione(prenotazioneDTO);

        log.info("END");
    }

    @Override
    public ListaPrenotazioneResponseDTO listaPrenotazioni(CoreRequestDTO request) {

        log.info("START");

        ListaPrenotazioneResponseDTO clientResponse = client.listaPrenotazioni(request);

        if (null == clientResponse) {
            log.debug("NULL RESPONSE FROM CLIENT");
            return null;
        }

        if (!clientResponse.isSuccess() || null == clientResponse.getContent()) {
            log.debug("some problem apped from client: {}", clientResponse.getError());
            return clientResponse;
        } else {

            for (PrenotazioneAppuntamentoDTO prenotazione : clientResponse.getContent().getPrenotazioni()){
                try{
                    PrenotazioneDTO prenotazioneFull = dao.getPrenotazione((long) prenotazione.getIdPrenotazione());
                    prenotazione.setT_satisfaction(prenotazioneFull.getT_satisfaction());
                }catch(Exception e){
                    log.debug("Prenotazione {} non trovata", prenotazione.getIdPrenotazione());
                }
            }

            log.info("END");
            return clientResponse;
        }

    }

    @Override
    public ListaPrenotazioneResponseDTO prenotazioneById(PrenotazioneByIdRequestDTO request) {

        log.info("START");
        var prenTest = false;

        CodiceFiscaleFruitoreAndUserDTO result = dao.getCfRichiedenteFruitore(request.getIdPrenotazione());

        var coreRequest = new PrenotazioneByIdRequestDTO();

        coreRequest.setCfUser(request.getCfUser());
        coreRequest.setCfFruitore(request.getCfFruitore());
        coreRequest.setIdPrenotazione(request.getIdPrenotazione());
		/*
		if(PREN_TEST) {
			coreRequest.setCfUser(request.getCfUser());
			coreRequest.setCfFruitore(request.getCfFruitore());
			coreRequest.setIdPrenotazione(request.getIdPrenotazione());
		} else {
			if (null==result) {
				log.info("nessuna corrispondenza tra di CF user per la prenotazione {}", request.getIdPrenotazione());
				return null;
			}
			coreRequest.setIdPrenotazione(request.getIdPrenotazione());
			coreRequest.setCfFruitore(result.getT_cf_fruitore().toUpperCase());
			coreRequest.setCfUser(result.getT_codice_fiscale().toUpperCase());
		}
		*/
        ListaPrenotazioneResponseDTO clientResponse = client.listaPrenotazioni(coreRequest);

        if (null == clientResponse) {
            log.debug("NULL RESPONSE FROM CLIENT");
            return null;
        }

        if (!clientResponse.isSuccess() || null == clientResponse.getContent()) {
            log.debug("some problem apped from client: {}", clientResponse.getError());
            return clientResponse;
        } else {

            ListaPrenotazioneResponseDTO response = new ListaPrenotazioneResponseDTO();
            ListaPrenotazioneContentDTO content = new ListaPrenotazioneContentDTO();
            List<PrenotazioneAppuntamentoDTO> newLista = new ArrayList<PrenotazioneAppuntamentoDTO>();
            PrenotazioneAppuntamentoDTO newAppuntamento;

            if (null != clientResponse.getContent().getPrenotazioni() && clientResponse.getContent().getPrenotazioni().size() > 0) {
                List<PrenotazioneAppuntamentoDTO> currentList = clientResponse.getContent().getPrenotazioni();
                for (PrenotazioneAppuntamentoDTO prenotazione : currentList)
                    if (prenotazione.getIdPrenotazione() == request.getIdPrenotazione()) {
                        newAppuntamento = prenotazione;
                        newLista.add(newAppuntamento);
                    }
            }
            content.setPrenotazioni(newLista);
            response.setContent(content);
            response.setSuccess(clientResponse.isSuccess());

            log.info("END");
            return response;
        }
    }

    @Override
    public ProfilatoreDettaglioUtenteResponseDTO getDettaglioUtenza(ProfilatoreDettaglioUtenteRequestDTO request) {

        log.info("START");

        ProfilatoreDettaglioUtenteResponseDTO responseClient = profilatoreClient.getDettaglioUtenza(request);

        if (null == responseClient || null == responseClient.getData()) {
            log.warn("NULL RESPONSE FROM CLIENT");
            return null;
        }

        log.info("END");

        return responseClient;
    }

    @Override
    public boolean sendPushGestorePratiche(GestorePraticaSendPushRequestDTO request) {
        log.info("START");
        boolean esito = dispositivoService.sendPushGestorePratiche(request);
        log.info("END");
        return esito;
    }


    @Override
    public boolean sendPushNews(String idNews) {
        log.info("START");
        boolean esito = dispositivoService.sendPushNews(idNews);
        log.info("END");
        return esito;
    }

	@Override
	public String getProperty(String property) {
		log.info("START");
		String prop;

		prop=env.getProperty(property);

		if(StringUtils.isBlank(prop)) {
			log.error("Proprietà non trovata");
			return null;
		}
		log.info("END");
		return prop;


	}

}
