package it.itsvil.macs.backend.service;


import it.itsvil.macs.backend.dto.*;
import it.itsvil.macs.backend.util.MacsEnum;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.RestTemplate;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;

import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;

@Log4j2
@Service(value = "ZoomServiceImpl")
public class ZoomServiceImpl implements  ZoomService{

	@Autowired
	private Environment env;

    // https://developers.zoom.us/docs/internal-apps/create/
	// https://marketplace.zoom.us/
	// ZoomMeetingServerOAuth

	@Value(value = "${zoomAccountId}")
	String zoomAccountId;
	@Value(value = "${zoomClientSecret}")
	String zoomClientSecret;
	@Value(value = "${zoomClientId}")
	String zoomClientId;
	@Value(value = "${zoomEmail}")
	String zoomEmail;
	@Value(value = "${zoomApiCreateURL}")
	String zoomApiCreateURL;
	@Value(value = "${zoomApiTokenURL}")
	String zoomApiTokenURL;


    @Override
	public ZoomMeetingObjectDTO  createMeeting(ZoomMeetingRequestDTO  zoomMeetingRequestDTO) throws Exception {

		if (zoomMeetingRequestDTO==null) return null;

		// PROPERTY CONFIGURATE
		String organizer = env.getProperty("macs.meeting.organizer");
		String subject = env.getProperty("macs.meeting.subject") + " " + zoomMeetingRequestDTO.getResourceName();
		// PROPERTY CONFIGURATE (SEPARATORE PIPE)
		String resourcesNames = env.getProperty("macs.meeting.resourcesNames");


		String[] resources = resourcesNames.split("\\|");
		boolean meeting_allow = false;
		for (String resource: resources){
			log.debug(resource+" "+zoomMeetingRequestDTO.getResourceName());
			if (resource.equalsIgnoreCase(zoomMeetingRequestDTO.getResourceName())){
				meeting_allow = true;
			}
		}
		if (meeting_allow==false) {
			log.debug(zoomMeetingRequestDTO.getResourceName()+ " non abilitata alla creazione meeting");
			return null;
		}


		ZoomMeetingObjectDTO  zoomMeetingObjectDTO = new ZoomMeetingObjectDTO();
		log.debug("Request to create a Zoom meeting...");
		// replace zoomUserId with your user ID
		String apiUrl = zoomApiCreateURL + "/users/" + zoomEmail + "/meetings";

		// replace with your password or method (non necessario)
		//zoomMeetingObjectDTO.setPassword(zoomPassword);
		// replace email with your email
		zoomMeetingObjectDTO.setHost_email(zoomEmail);

		// Optional Settings for host and participant related options
		ZoomMeetingSettingsDTO settingsDTO = new ZoomMeetingSettingsDTO();
		settingsDTO.setJoin_before_host(false);
		settingsDTO.setParticipant_video(true);
		settingsDTO.setHost_video(false);
		settingsDTO.setAuto_recording("cloud");
		settingsDTO.setMute_upon_entry(true);
		settingsDTO.setAllow_multiple_devices(true);
		settingsDTO.setWaiting_room(true);

		settingsDTO.setContact_name(organizer);

		List<ZoomInterpreterDTO> partecipanti = new ArrayList<ZoomInterpreterDTO>();
		for (String email: zoomMeetingRequestDTO.getParticipants()){
			ZoomInterpreterDTO part = new ZoomInterpreterDTO();
			part.setEmail(email);
			partecipanti.add(part);
		}
		settingsDTO.setInterpreters(partecipanti);

		zoomMeetingObjectDTO.setSettings(settingsDTO);

		zoomMeetingObjectDTO.setTopic(zoomMeetingRequestDTO.getResourceName());

		LocalDate localDate = LocalDate.of( Integer.parseInt(zoomMeetingRequestDTO.getDay().split("-")[0]),
				Integer.parseInt(zoomMeetingRequestDTO.getDay().split("-")[1]),
				Integer.parseInt(zoomMeetingRequestDTO.getDay().split("-")[2]));
		LocalTime localTime = LocalTime.of( Integer.parseInt(zoomMeetingRequestDTO.getStart().split(":")[0]),
				Integer.parseInt(zoomMeetingRequestDTO.getStart().split(":")[1]),
				0);
		LocalDateTime localDateTime = LocalDateTime.of(localDate, localTime);

		zoomMeetingObjectDTO.setStart_time(localDateTime.toString()+":00Z");
		zoomMeetingObjectDTO.setDuration(zoomMeetingRequestDTO.getDuration());

		String token = generateTOKEN();

		RestTemplate restTemplate = new RestTemplate();
		HttpHeaders headers = new HttpHeaders();
		headers.add("Authorization", "Bearer "+token);
		headers.add("Content-Type", "application/json");

		HttpEntity<ZoomMeetingObjectDTO> httpEntity = new HttpEntity<ZoomMeetingObjectDTO>(zoomMeetingObjectDTO, headers);

		ResponseEntity<ZoomMeetingObjectDTO> zEntity = restTemplate.exchange(apiUrl, HttpMethod.POST, httpEntity, ZoomMeetingObjectDTO.class);
		if(zEntity.getStatusCodeValue() >= 200 && zEntity.getStatusCodeValue() < 300) {
			log.debug("Zooom meeeting response {}",zEntity);

			// NOTIFICATION
			{
				ZoomMeetingEmailRequestDTO zoomMeetingEmailRequestDTO = new ZoomMeetingEmailRequestDTO();
				zoomMeetingEmailRequestDTO.setTypeBody(MacsEnum.MACS_TYPE_BODY_MAIL.getCodice());
				// adeguamento nome property sbagliata del servizio che verrà chiamato
				zoomMeetingEmailRequestDTO.setTextOgetto(subject);

				List<String> destinatari = new ArrayList();
				for (String email: zoomMeetingRequestDTO.getParticipants()){
					destinatari.add(email);
				}
				zoomMeetingEmailRequestDTO.setDestinatari(destinatari);

				SimpleDateFormat format = new SimpleDateFormat("dd/MM/yyyy");
				String dateString = format.format( java.sql.Date.valueOf(zEntity.getBody().getStart_time().split("T")[0]) );
				String hourString = zEntity.getBody().getStart_time().split("T")[1].substring(0,5);

				List<String> placeHolder = new ArrayList();
				placeHolder.add(organizer);
				placeHolder.add(zoomMeetingRequestDTO.getResourceName());
				placeHolder.add(dateString + " ore " + hourString);
				placeHolder.add(zEntity.getBody().getJoin_url());
				placeHolder.add(zEntity.getBody().getJoin_url());

				zoomMeetingEmailRequestDTO.setPlaceHolder(placeHolder);

				String mittente = env.getProperty("macs.meeting.sender");
				String alias = env.getProperty("macs.meeting.alias");

				zoomMeetingEmailRequestDTO.setMittente(mittente);
				zoomMeetingEmailRequestDTO.setAlias(alias);

				sendMailNotification(zoomMeetingEmailRequestDTO);
			}

			return zEntity.getBody();

		} else {
			log.debug("Error while creating zoom meeting {}", zEntity.getStatusCode());
			throw new Exception(zEntity.getStatusCode()+"");
		}

	}




	@Override
	public String  deleteMeeting(String id_meeting) throws Exception {

		String apiUrl = zoomApiCreateURL + "/meetings/" + id_meeting;

		String token = generateTOKEN();

		RestTemplate restTemplate = new RestTemplate();
		HttpHeaders headers = new HttpHeaders();
		headers.add("Authorization", "Bearer "+token);
		headers.add("Content-Type", "application/json");

		HttpEntity httpEntity = new HttpEntity(headers);

		try {
			ResponseEntity zEntity = restTemplate.exchange(apiUrl, HttpMethod.DELETE, httpEntity, ZoomMeetingObjectDTO.class);
			if (zEntity.getStatusCodeValue() >= 200 && zEntity.getStatusCodeValue() < 300) {
				log.debug("Zooom meeeting response {}", zEntity);

				// NOTIFICATION
				/*
				{
					ZoomMeetingEmailRequestDTO zoomMeetingEmailRequestDTO = new ZoomMeetingEmailRequestDTO();
					zoomMeetingEmailRequestDTO.setTypeBody(MacsEnum.MACS_TYPE_BODY_MAIL.getCodice());
					// adeguamento nome property sbagliata del servizio che verrà chiamato
					zoomMeetingEmailRequestDTO.setTextOgetto(subject);

					List<String> destinatari = new ArrayList();
					for (String email: zoomMeetingRequestDTO.getParticipants()){
						destinatari.add(email);
					}
					zoomMeetingEmailRequestDTO.setDestinatari(destinatari);

					SimpleDateFormat format = new SimpleDateFormat("dd/MM/yyyy");
					String dateNow = format.format( new java.sql.Date(System.currentTimeMillis()) );
					String dateString = format.format( java.sql.Date.valueOf(zEntity.getBody().getStart_time().split("T")[0]) );
					String hourString = zEntity.getBody().getStart_time().split("T")[1].substring(0,5);

					List<String> placeHolder = new ArrayList();
					placeHolder.add(dateNow);
					placeHolder.add(organizer);
					placeHolder.add(zoomMeetingRequestDTO.getResourceName());
					placeHolder.add(dateString + " ore " + hourString);
					placeHolder.add(zEntity.getBody().getJoin_url());
					placeHolder.add(zEntity.getBody().getJoin_url());

					zoomMeetingEmailRequestDTO.setPlaceHolder(placeHolder);

					sendMailNotification(zoomMeetingEmailRequestDTO);
				}
				*/

				return "Correttamente eliminata";

			} else {
				log.debug("Error while deleting zoom meeting {}", zEntity.getStatusCode());
				return "Non eliminata";
			}

		}catch(HttpClientErrorException ee){
			return ee.getMessage();
		}

	}


	private String generateTOKEN() {
		String basicAuthEncoded = Base64.getEncoder().encodeToString((zoomClientId + ":" + zoomClientSecret).getBytes());

		String apiUrl = zoomApiTokenURL + zoomAccountId;

		RestTemplate restTemplate = new RestTemplate();
		HttpHeaders headers = new HttpHeaders();
		headers.add("Authorization", "Basic "+basicAuthEncoded);
		headers.add("Host", "zoom.us");

		ResponseEntity<Map> zEntity = restTemplate.exchange(apiUrl, HttpMethod.POST, new HttpEntity(headers), Map.class);

		return (String) zEntity.getBody().get("access_token");
	}



	private void sendMailNotification(ZoomMeetingEmailRequestDTO zoomMeetingEmailRequestDTO) {

		String apiUrl = env.getProperty("macs.meeting.sendEmail.url");
		String applicationId = env.getProperty("macs.notificatore.applicationId");

		RestTemplate restTemplate = new RestTemplate();
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/json");
		headers.add("ApplicationId", applicationId);

		HttpEntity<ZoomMeetingEmailRequestDTO> httpEntity = new HttpEntity<ZoomMeetingEmailRequestDTO>(zoomMeetingEmailRequestDTO, headers);

		ResponseEntity<?> zEntity = restTemplate.exchange(apiUrl, HttpMethod.POST, httpEntity, Object.class);
		if(zEntity.getStatusCodeValue() >= 200 && zEntity.getStatusCodeValue() < 300) {
			log.debug("Zooom meeeting mail sent" );
		} else {
			log.debug("Error while creating zoom mail meeting {}", zEntity.getStatusCode());
		}

	}



}
