package it.itsvil.macs.configuration;

import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.mapper.MapperFactoryBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.env.Environment;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.jdbc.datasource.DriverManagerDataSource;
import org.springframework.transaction.PlatformTransactionManager;

import it.itsvil.macs.backend.mapper.DispositivoMapper;
import it.itsvil.macs.backend.mapper.MacsMapper;
import it.itsvil.macs.backend.mapper.UserMapper;
import it.itsvil.macs.backend.mapper.ComunicazioneMapper;
import it.itsvil.macs.backend.mapper.ConfigurationMacsMapper;

@Configuration
public class MybatisConfig {

	public final String MAPPER_LOCATIONS_PATH_MACS = "mapper/MacsMapper.xml";
	public final String MAPPER_LOCATIONS_PATH_USER = "mapper/UserMapper.xml";
	public final String MAPPER_LOCATIONS_PATH_DISPOSITIVO = "mapper/DispositivoMapper.xml";
	public final String MAPPER_LOCATIONS_PATH_COMUNICAZIONE = "mapper/ComunicazioneMapper.xml";
	public final String MAPPER_LOCATIONS_PATH_CONFIGURAZIONE = "mapper/ConfigurationMacsMapper.xml";
	public final String TYPE_ALIASES_PACKAGE = "it.itsvil.macs.backend.mapper";
	public final String CONFIG_FILE_NAME = "mybatis-config.xml";     
	    
    @Autowired
    private Environment env;
    
    @Primary
    @Bean
    public DataSource getDataSource() throws NamingException {

    	DriverManagerDataSource dataSource = new DriverManagerDataSource();
        dataSource.setDriverClassName(env.getProperty("spring.datasource.driverClassName"));
        dataSource.setUrl(env.getProperty("spring.datasource.url"));
        dataSource.setUsername(env.getProperty("spring.datasource.username"));
        dataSource.setPassword(env.getProperty("spring.datasource.password"));
        return dataSource;
    }
    
    @Primary
    @Bean
    public SqlSessionFactory sqlSessionFactoryBeanMacsMapper() throws Exception {
        SqlSessionFactoryBean sqlSessionFactoryBean = new SqlSessionFactoryBean();
        sqlSessionFactoryBean.setDataSource(getDataSource());

        PathMatchingResourcePatternResolver resolver = new PathMatchingResourcePatternResolver();
        sqlSessionFactoryBean.setMapperLocations(resolver.getResources(MAPPER_LOCATIONS_PATH_MACS));        
        sqlSessionFactoryBean.setTypeAliasesPackage(TYPE_ALIASES_PACKAGE);
        sqlSessionFactoryBean.setConfigLocation(new ClassPathResource(CONFIG_FILE_NAME));
        return sqlSessionFactoryBean.getObject();
    }
    
    @Bean
    public SqlSessionFactory sqlSessionFactoryBeanUserMapper() throws Exception {
        SqlSessionFactoryBean sqlSessionFactoryBean = new SqlSessionFactoryBean();
        sqlSessionFactoryBean.setDataSource(getDataSource());

        PathMatchingResourcePatternResolver resolver = new PathMatchingResourcePatternResolver();
        sqlSessionFactoryBean.setMapperLocations(resolver.getResources(MAPPER_LOCATIONS_PATH_USER));        
        sqlSessionFactoryBean.setTypeAliasesPackage(TYPE_ALIASES_PACKAGE);
        sqlSessionFactoryBean.setConfigLocation(new ClassPathResource(CONFIG_FILE_NAME));
        return sqlSessionFactoryBean.getObject();
    }
    
    @Bean
    public SqlSessionFactory sqlSessionFactoryBeanDispositivoMapper() throws Exception {
        SqlSessionFactoryBean sqlSessionFactoryBean = new SqlSessionFactoryBean();
        sqlSessionFactoryBean.setDataSource(getDataSource());

        PathMatchingResourcePatternResolver resolver = new PathMatchingResourcePatternResolver();
        sqlSessionFactoryBean.setMapperLocations(resolver.getResources(MAPPER_LOCATIONS_PATH_DISPOSITIVO));        
        sqlSessionFactoryBean.setTypeAliasesPackage(TYPE_ALIASES_PACKAGE);
        sqlSessionFactoryBean.setConfigLocation(new ClassPathResource(CONFIG_FILE_NAME));
        return sqlSessionFactoryBean.getObject();
    }
    
    @Bean
    public SqlSessionFactory sqlSessionFactoryBeanComunicazioneMapper() throws Exception {
        SqlSessionFactoryBean sqlSessionFactoryBean = new SqlSessionFactoryBean();
        sqlSessionFactoryBean.setDataSource(getDataSource());

        PathMatchingResourcePatternResolver resolver = new PathMatchingResourcePatternResolver();
        sqlSessionFactoryBean.setMapperLocations(resolver.getResources(MAPPER_LOCATIONS_PATH_COMUNICAZIONE));        
        sqlSessionFactoryBean.setTypeAliasesPackage(TYPE_ALIASES_PACKAGE);
        sqlSessionFactoryBean.setConfigLocation(new ClassPathResource(CONFIG_FILE_NAME));
        return sqlSessionFactoryBean.getObject();
    }
    
    @Bean
    public SqlSessionFactory sqlSessionFactoryBeanConfigurazioneMacs() throws Exception {
        SqlSessionFactoryBean sqlSessionFactoryBean = new SqlSessionFactoryBean();
        sqlSessionFactoryBean.setDataSource(getDataSource());

        PathMatchingResourcePatternResolver resolver = new PathMatchingResourcePatternResolver();
        sqlSessionFactoryBean.setMapperLocations(resolver.getResources(MAPPER_LOCATIONS_PATH_CONFIGURAZIONE));        
        sqlSessionFactoryBean.setTypeAliasesPackage(TYPE_ALIASES_PACKAGE);
        sqlSessionFactoryBean.setConfigLocation(new ClassPathResource(CONFIG_FILE_NAME));
        return sqlSessionFactoryBean.getObject();
    }
    
    @Primary
    @Bean
    public PlatformTransactionManager transactionManager() throws NamingException {
    	 DataSourceTransactionManager transactionManager = new DataSourceTransactionManager(getDataSource());
         transactionManager.setGlobalRollbackOnParticipationFailure(true);
         return transactionManager;
    }
    
    @Primary
    @Bean
	public MapperFactoryBean<MacsMapper> macsMapper() throws Exception {
	  MapperFactoryBean<MacsMapper> factoryBeanMacsMapper = new MapperFactoryBean<>(MacsMapper.class);
	  factoryBeanMacsMapper.setSqlSessionFactory(sqlSessionFactoryBeanMacsMapper());
	  return factoryBeanMacsMapper;
	}
    
    @Primary
    @Bean
	public MapperFactoryBean<DispositivoMapper> dispositivoMapper() throws Exception {
	  MapperFactoryBean<DispositivoMapper> factoryBeanDispositivoMapper = new MapperFactoryBean<>(DispositivoMapper.class);
	  factoryBeanDispositivoMapper.setSqlSessionFactory(sqlSessionFactoryBeanDispositivoMapper());
	  return factoryBeanDispositivoMapper;
	}
    
    @Primary
    @Bean
	public MapperFactoryBean<ComunicazioneMapper> ComunicazioneMapper() throws Exception {
	  MapperFactoryBean<ComunicazioneMapper> factoryBeanComunicazioneMapper = new MapperFactoryBean<>(ComunicazioneMapper.class);
	  factoryBeanComunicazioneMapper.setSqlSessionFactory(sqlSessionFactoryBeanComunicazioneMapper());
	  return factoryBeanComunicazioneMapper;
	}
    
    @Primary
    @Bean
	public MapperFactoryBean<UserMapper> userMapper() throws Exception {
	  MapperFactoryBean<UserMapper> factoryBeanUserMapper = new MapperFactoryBean<>(UserMapper.class);
	  factoryBeanUserMapper.setSqlSessionFactory(sqlSessionFactoryBeanUserMapper());
	  return factoryBeanUserMapper;
	}
    
    @Primary
    @Bean
	public MapperFactoryBean<ConfigurationMacsMapper> configurazioneMacsMapper() throws Exception {
	  MapperFactoryBean<ConfigurationMacsMapper> factoryBeanUserMapper = new MapperFactoryBean<>(ConfigurationMacsMapper.class);
	  factoryBeanUserMapper.setSqlSessionFactory(sqlSessionFactoryBeanConfigurazioneMacs());
	  return factoryBeanUserMapper;
	}
}
