package it.itsvil.macs.rest;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import it.itsvil.macs.backend.dto.ComunicazioneDTO;
import it.itsvil.macs.backend.dto.ComunicazioneRequestDTO;
import it.itsvil.macs.backend.dto.RisultatoDTO;
import it.itsvil.macs.backend.service.ComunicazioneService;
import lombok.extern.log4j.Log4j2;

@Log4j2
@RestController
@RequestMapping(value= "/comunicazione")
public class ComunicazioneServiceRest {
	
	@Autowired
	private ComunicazioneService service;
	
	@GetMapping(value = "/1.0/getComunicazioneById" , produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<ComunicazioneDTO>> getComunicazioneById(@RequestParam Long id){
		RisultatoDTO<ComunicazioneDTO> response = new RisultatoDTO<ComunicazioneDTO>();
		try {
			log.info("START");
			
			if (null==id) {
				response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
				log.info("END WITH BAD REQUEST");
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}else {
				ComunicazioneDTO result  = service.getComunicazioneById(id);
				if (null==result) {
					response.setData(result);
					response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
					log.info("END WITH EXPECTION FAIL");
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}else {
					response.setData(result);
					response.success(HttpStatus.SC_OK);
				}
			}
		}catch (Exception e) {
			response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
			log.error("END WITH SERVER ERROR: {}", e.getMessage());
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		log.info("END");
		return ResponseEntity.ok(response);
	}
	
	@PostMapping(value = "/1.0/insertComunicazione" , consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<Long>> insertComunicazione(@RequestBody ComunicazioneRequestDTO request){
		RisultatoDTO<Long> response = new RisultatoDTO<Long>();
		try {
			log.info("START");
			if (null==request || null==request.getT_allegato()|| null==request.getT_descrizione_breve() || null==request.getT_testo_completo() || null==request.getT_titolo()
					|| StringUtils.isBlank(request.getT_allegato()) ||  StringUtils.isBlank(request.getT_descrizione_breve()) ||  StringUtils.isBlank(request.getT_testo_completo())
					||  StringUtils.isBlank(request.getT_titolo())){
				response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
				log.info("END WITH BAD REQUEST");
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}else {
				Long risultato=service.insertComunicazione(request);
				if (null== risultato) {
					response.setData(risultato);
					response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
					log.info("END WITH EXPECTION FAIL");
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
				response.setData(risultato);
				response.success(HttpStatus.SC_OK);
				log.info("END");				
			}
		}catch (Exception e) {
			response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
			log.error("END WITH SERVER ERROR: {}"+e.getMessage());
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
}
