package it.itsvil.macs.rest;

import com.google.gson.Gson;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import it.itsvil.macs.backend.dto.AnnullaPrenotazioneRequestDTO;
import it.itsvil.macs.backend.dto.AnnullaPrenotazioneResponseDTO;
import it.itsvil.macs.backend.dto.AuditRequestDTO;
import it.itsvil.macs.backend.dto.ConfermaPrenotazioneCustomRequestDTO;
import it.itsvil.macs.backend.dto.ConfermaPrenotazioneResponseDTO;
import it.itsvil.macs.backend.dto.CoreRequestDTO;
import it.itsvil.macs.backend.dto.DisdiciPrenotazioneRequestDTO;
import it.itsvil.macs.backend.dto.DisdiciPrenotazioneResponseDTO;
import it.itsvil.macs.backend.dto.DownloadDocumentoRequestDTO;
import it.itsvil.macs.backend.dto.DownloadDocumentoResponseDTO;
import it.itsvil.macs.backend.dto.GestorePraticaSendPushRequestDTO;
import it.itsvil.macs.backend.dto.InfoRisorseRequestDTO;
import it.itsvil.macs.backend.dto.InfoRisorseResponseDTO;
import it.itsvil.macs.backend.dto.InitPrenotazioneRequestDTO;
import it.itsvil.macs.backend.dto.InitPrenotazioneResponseDTO;
import it.itsvil.macs.backend.dto.ListaPrenotazioneResponseDTO;
import it.itsvil.macs.backend.dto.ListaRisorseMobileAppRequest;
import it.itsvil.macs.backend.dto.ListaRisorseMobileAppResponse;
import it.itsvil.macs.backend.dto.PeriodiDisponibiliRequestDTO;
import it.itsvil.macs.backend.dto.PeriodiDisponibiliResponseDTO;
import it.itsvil.macs.backend.dto.PrenotazioneByIdRequestDTO;
import it.itsvil.macs.backend.dto.ProfilatoreDettaglioUtenteDTO;
import it.itsvil.macs.backend.dto.ProfilatoreDettaglioUtenteRequestDTO;
import it.itsvil.macs.backend.dto.ProfilatoreDettaglioUtenteResponseDTO;
import it.itsvil.macs.backend.dto.ProviderListResponseDTO;
import it.itsvil.macs.backend.dto.RisultatoDTO;
import it.itsvil.macs.backend.service.MacsService;
import lombok.extern.log4j.Log4j2;

@Log4j2
@RestController
@RequestMapping(value = "/service")
public class MacsServiceRest {

    @Autowired
    private MacsService service;

    @GetMapping(value = "/1.0/getKeepAlive", produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<String>> getKeepAlive(@RequestParam int id) {
        RisultatoDTO<String> response = new RisultatoDTO<String>();
        String keep = service.keepAlive(id);
        response.setData(keep);
        response.success(HttpStatus.SC_OK);
        return ResponseEntity.ok(response);
    }

    @PostMapping(value = "/1.0/prenotazioneById", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<ListaPrenotazioneResponseDTO>> prenotazioneById(@RequestBody PrenotazioneByIdRequestDTO request) {
        RisultatoDTO<ListaPrenotazioneResponseDTO> response = new RisultatoDTO<ListaPrenotazioneResponseDTO>();
        try {
            log.info("START");
            if (null == request /*|| null==request.getCfFruitore()|| null==request.getCfUser() || StringUtils.isBlank(request.getCfFruitore()) ||  StringUtils.isBlank(request.getCfUser())*/
                    || request.getIdPrenotazione() <= 0) {
                response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
                log.info("END WITH BAD REQUEST");
                return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
            } else {
                ListaPrenotazioneResponseDTO risultato = service.prenotazioneById(request);
                if (null == risultato || !risultato.isSuccess()) {
                    response.setData(risultato);
                    response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
                    log.info("END WITH EXPECTION FAIL");
                    return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
                }
                response.setData(risultato);
                response.success(HttpStatus.SC_OK);
                log.info("END");
            }
        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        return ResponseEntity.ok(response);
    }

    @PostMapping(value = "/1.0/audit", consumes = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public void audit(@RequestBody AuditRequestDTO request) {
        log.debug("START");
        try {
            if (null != request)
                service.audit(request);
        } catch (Exception e) {
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
        }
        log.debug("END");
    }


    @PostMapping(value = "/1.0/listaUffici", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<ListaRisorseMobileAppResponse>> listaUffici(@RequestBody CoreRequestDTO request) {
        RisultatoDTO<ListaRisorseMobileAppResponse> response = new RisultatoDTO<ListaRisorseMobileAppResponse>();
        try {
            log.info("START");
            if (null == request || null == request.getCfFruitore() || null == request.getCfUser() || StringUtils.isBlank(request.getCfFruitore()) || StringUtils.isBlank(request.getCfUser())) {
                response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
                log.info("END WITH BAD REQUEST");
                return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
            } else {
                ListaRisorseMobileAppResponse risultato = service.listaUffici(request);
                if (null == risultato || !risultato.isSuccess()) {
                    response.setData(risultato);
                    response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
                    log.info("END WITH EXPECTION FAIL");
                    return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
                }
                response.setData(risultato);
                response.success(HttpStatus.SC_OK);
                log.info("END");
            }
        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        return ResponseEntity.ok(response);
    }

    @PostMapping(value = "/1.0/listaRisorse", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<ListaRisorseMobileAppResponse>> listaRisorse(@RequestBody ListaRisorseMobileAppRequest request) {
        RisultatoDTO<ListaRisorseMobileAppResponse> response = new RisultatoDTO<ListaRisorseMobileAppResponse>();
        try {
            log.info("START");
            if (null == request || null == request.getCfFruitore() || null == request.getCfUser() || StringUtils.isBlank(request.getCfFruitore()) || StringUtils.isBlank(request.getCfUser()) || 0 >= request.getId()) {
                response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
                log.info("END WITH BAD REQUEST");
                return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
            } else {
                ListaRisorseMobileAppResponse risultato = service.listaRisorsa(request);
                if (null == risultato || !risultato.isSuccess()) {
                    response.setData(risultato);
                    response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
                    log.info("END WITH EXPECTION FAIL");
                    return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
                }
                response.setData(risultato);
                response.success(HttpStatus.SC_OK);
                log.info("END");
            }
        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        return ResponseEntity.ok(response);
    }

    @PostMapping(value = "/1.0/infoRisorse", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<InfoRisorseResponseDTO>> infoRisorse(@RequestBody InfoRisorseRequestDTO request) {
        RisultatoDTO<InfoRisorseResponseDTO> response = new RisultatoDTO<InfoRisorseResponseDTO>();
        try {
            log.info("START");
            if (null == request || null == request.getCfFruitore() || null == request.getCfUser() || StringUtils.isBlank(request.getCfFruitore()) || StringUtils.isBlank(request.getCfUser()) || 0 >= request.getIdRisorse()) {
                response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
                log.info("END WITH BAD REQUEST");
                return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
            } else {
                InfoRisorseResponseDTO risultato = service.infoRisorse(request);
                if (null == risultato || !risultato.isSuccess()) {
                    response.setData(risultato);
                    response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
                    log.info("END WITH EXPECTION FAIL");
                    return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
                }
                response.setData(risultato);
                response.success(HttpStatus.SC_OK);
                log.info("END");
            }
        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        return ResponseEntity.ok(response);
    }

    @PostMapping(value = "/1.0/periodiDisponibili", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<PeriodiDisponibiliResponseDTO>> periodiDisponibili(@RequestBody PeriodiDisponibiliRequestDTO request) {
        RisultatoDTO<PeriodiDisponibiliResponseDTO> response = new RisultatoDTO<PeriodiDisponibiliResponseDTO>();
        try {
            log.info("START");
            if (null == request || null == request.getCfFruitore() || null == request.getCfUser() || StringUtils.isBlank(request.getCfFruitore()) || StringUtils.isBlank(request.getCfUser()) || 0 >= request.getIdRisorsa() || null == request.getDa() || StringUtils.isBlank(request.getDa())) {
                response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
                log.info("END WITH BAD REQUEST");
                return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
            } else {
                PeriodiDisponibiliResponseDTO risultato = service.periodiDisponibili(request);
                if (null == risultato || !risultato.isSuccess()) {
                    response.setData(risultato);
                    response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
                    log.info("END WITH EXPECTION FAIL");
                    return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
                }
                response.setData(risultato);
                response.success(HttpStatus.SC_OK);
                log.info("END");
            }
        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        return ResponseEntity.ok(response);
    }

    @PostMapping(value = "/1.0/downloadDocumento", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<DownloadDocumentoResponseDTO>> downloadDocumento(@RequestBody DownloadDocumentoRequestDTO request) {
        RisultatoDTO<DownloadDocumentoResponseDTO> response = new RisultatoDTO<DownloadDocumentoResponseDTO>();
        try {
            log.info("START");
            if (null == request || null == request.getCfFruitore() || null == request.getCfUser() || StringUtils.isBlank(request.getCfFruitore()) || StringUtils.isBlank(request.getCfUser()) || 0 >= request.getIdDocumento()) {
                response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
                log.info("END WITH BAD REQUEST");
                return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
            } else {
                DownloadDocumentoResponseDTO risultato = service.downloadDocumento(request);
                if (null == risultato || !risultato.isSuccess()) {
                    response.setData(risultato);
                    response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
                    log.info("END WITH EXPECTION FAIL");
                    return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
                }
                response.setData(risultato);
                response.success(HttpStatus.SC_OK);
                log.info("END");
            }
        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        return ResponseEntity.ok(response);
    }

    @PostMapping(value = "/1.0/initPrenotazione", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<InitPrenotazioneResponseDTO>> initPrenotazione(@RequestBody InitPrenotazioneRequestDTO request) {
        RisultatoDTO<InitPrenotazioneResponseDTO> response = new RisultatoDTO<InitPrenotazioneResponseDTO>();
        try {
            log.info("START");
            if (null == request || StringUtils.isBlank(request.getCfFruitore()) || StringUtils.isBlank(request.getCfUser()) || null == request.getSlots() || 0 > request.getSlots().size()) {
                response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
                log.info("END WITH BAD REQUEST");
                return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
            } else {
                InitPrenotazioneResponseDTO risultato = service.initPrenotazione(request);
                if (null == risultato || !risultato.isSuccess()) {
                    response.setData(risultato);
                    response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
                    log.info("END WITH EXPECTION FAIL");
                    return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
                }
                response.setData(risultato);
                response.success(HttpStatus.SC_OK);
                log.info("END");
            }
        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        return ResponseEntity.ok(response);
    }

    @PostMapping(value = "/1.0/anullaPrenotazione", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<AnnullaPrenotazioneResponseDTO>> anullaPrenotazione(@RequestBody AnnullaPrenotazioneRequestDTO request) {
        RisultatoDTO<AnnullaPrenotazioneResponseDTO> response = new RisultatoDTO<AnnullaPrenotazioneResponseDTO>();
        try {
            log.info("START");
            if (null == request || StringUtils.isBlank(request.getCfFruitore()) || StringUtils.isBlank(request.getCfUser()) || StringUtils.isBlank(request.getTokenPrenotazione())) {
                response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
                log.info("END WITH BAD REQUEST");
                return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
            } else {
                AnnullaPrenotazioneResponseDTO risultato = service.annullaPrenotazione(request);
                if (null == risultato || !risultato.isSuccess()) {
                    response.setData(risultato);
                    response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
                    log.info("END WITH EXPECTION FAIL");
                    return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
                }
                response.setData(risultato);
                response.success(HttpStatus.SC_OK);
                log.info("END");
            }
        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        return ResponseEntity.ok(response);
    }

    @PostMapping(value = "/1.0/confermaPrenotazione", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<ConfermaPrenotazioneResponseDTO>> confermaPrenotazione(@RequestBody ConfermaPrenotazioneCustomRequestDTO request) {
        RisultatoDTO<ConfermaPrenotazioneResponseDTO> response = new RisultatoDTO<ConfermaPrenotazioneResponseDTO>();
        try {
            log.info("START");
            if (null == request || null == request.getConfermaPrenotazione() || null == request.getConfermaPrenotazione().getCfFruitore() || null == request.getConfermaPrenotazione().getCfUser()
                    || StringUtils.isBlank(request.getConfermaPrenotazione().getCfFruitore()) || StringUtils.isBlank(request.getConfermaPrenotazione().getCfUser())
                    || StringUtils.isBlank(request.getConfermaPrenotazione().getTokenPrenotazione())
                    || null == request.getConfermaPrenotazione().getAnagraficaFruitore() || null == request.getConfermaPrenotazione().getAnagraficaPresentante()
                    /*|| null==request.getConfermaPrenotazione().getRicevuta()*/ || null == request.getDataInizioPrenotazione() || request.getIdRisorsa() <= 0 || request.getIdUfficio() <= 0
                    || StringUtils.isBlank(request.getNomeRisorsa()) || StringUtils.isBlank(request.getNomeUfficio())
                    || StringUtils.isBlank(request.getPrenotazionePersonale()) || StringUtils.isBlank(request.getSpidCode())) {
                response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
                log.info("END WITH BAD REQUEST");
                return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
            } else {
                ConfermaPrenotazioneResponseDTO risultato = service.confermaPrenotazione(request);
                if (null == risultato || !risultato.isSuccess()) {
                    response.setData(risultato);
                    response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
                    log.info("END WITH EXPECTION FAIL");
                    return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
                }
                response.setData(risultato);
                response.success(HttpStatus.SC_OK);
                log.info("END");
            }
        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        return ResponseEntity.ok(response);
    }

    @PostMapping(value = "/1.0/disdiciPrenotazione", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<DisdiciPrenotazioneResponseDTO>> disdiciPrenotazione(@RequestBody DisdiciPrenotazioneRequestDTO request) {
        RisultatoDTO<DisdiciPrenotazioneResponseDTO> response = new RisultatoDTO<DisdiciPrenotazioneResponseDTO>();
        try {
            log.info("START");
            if (null == request || StringUtils.isBlank(request.getCfFruitore()) || StringUtils.isBlank(request.getCfUser()) || 0 >= (request.getIdPrenotazione())) {
                response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
                log.info("END WITH BAD REQUEST");
                return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
            } else {
                DisdiciPrenotazioneResponseDTO risultato = service.disdiciPrenotazione(request);
                if (null == risultato || !risultato.isSuccess()) {
                    response.setData(risultato);
                    response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
                    log.info("END WITH EXPECTION FAIL");
                    return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
                }
                response.setData(risultato);
                response.success(HttpStatus.SC_OK);
                log.info("END");
            }
        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        return ResponseEntity.ok(response);
    }

    @PostMapping(value = "/1.0/updateSatisfaction", produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<Boolean>> updateSatisfaction(@RequestParam(value = "n_id_prenotazione", name = "n_id_prenotazione") Integer n_id_prenotazione, @RequestParam(value = "t_satisfaction", name = "t_satisfaction")  String t_satisfaction) {
        log.debug("START");
        RisultatoDTO<Boolean> response = new RisultatoDTO<Boolean>();
        try {

                service.updateSatisfactionPrenotazione(n_id_prenotazione, t_satisfaction);

                response.setData(true);
                response.success(HttpStatus.SC_OK);
                log.info("END");

        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        log.debug("END");
        return ResponseEntity.ok(response);
    }



    @PostMapping(value = "/1.0/listaPrenotazioni", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<ListaPrenotazioneResponseDTO>> listaPrenotazioni(@RequestBody CoreRequestDTO request) {
        RisultatoDTO<ListaPrenotazioneResponseDTO> response = new RisultatoDTO<ListaPrenotazioneResponseDTO>();
        try {
            log.info("START");
            if (null == request || StringUtils.isBlank(request.getCfFruitore()) || StringUtils.isBlank(request.getCfUser())) {
                response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
                log.info("END WITH BAD REQUEST");
                return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
            } else {
                ListaPrenotazioneResponseDTO risultato = service.listaPrenotazioni(request);
                if (null == risultato || !risultato.isSuccess()) {
                    response.setData(risultato);
                    response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
                    log.info("END WITH EXPECTION FAIL");
                    return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
                }
                response.setData(risultato);
                response.success(HttpStatus.SC_OK);
                log.info("END");
            }
        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        return ResponseEntity.ok(response);
    }

    //TODO da valutare se continuare ad usarlo o no
    @CrossOrigin
    @GetMapping(value = "/1.0/listaProvider", produces = {MediaType.APPLICATION_JSON_VALUE})
    public ResponseEntity<RisultatoDTO<ProviderListResponseDTO>> listaProvider() {

        RisultatoDTO<ProviderListResponseDTO> response = new RisultatoDTO<ProviderListResponseDTO>();
        try {
            log.info("START");
            ProviderListResponseDTO risultato = service.providerList();
            if (null == risultato) {
                response.setData(risultato);
                response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
                log.info("END WITH EXPECTION FAIL");
                return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
            }
            response.setData(risultato);
            response.success(HttpStatus.SC_OK);
            log.info("END");
        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        return ResponseEntity.ok(response);
    }

    @GetMapping(value = "/1.0/getDettaglioUtenza", produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<ProfilatoreDettaglioUtenteDTO>> getDettaglioUtenza(@RequestParam String t_id_user) {

        RisultatoDTO<ProfilatoreDettaglioUtenteDTO> response = new RisultatoDTO<ProfilatoreDettaglioUtenteDTO>();
        try {
            log.info("START");
            if (null == t_id_user || StringUtils.isBlank(t_id_user)) {
                response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
                log.info("END WITH BAD REQUEST");
                return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
            } else {
                ProfilatoreDettaglioUtenteRequestDTO request = new ProfilatoreDettaglioUtenteRequestDTO();
                request.setT_id_user(t_id_user);
                ProfilatoreDettaglioUtenteResponseDTO result = service.getDettaglioUtenza(request);
                if (null == result || !result.isSuccess()) {
                    response.setData(result.getData());
                    response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
                    log.info("END WITH EXPECTION FAIL");
                    return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
                }
                response.setData(result.getData());
                response.success(HttpStatus.SC_OK);
                log.info("END");
            }

        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        return ResponseEntity.ok(response);
    }

    @PostMapping(value = "/1.0/notificaGestorePratica", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<String>> notificaGestorePratica(@RequestBody GestorePraticaSendPushRequestDTO request) {
        RisultatoDTO<String> response = new RisultatoDTO<String>();
        try {

            log.info("START");
            if (null == request || null == request.getSpidCode() || null == request.getCodiceFiscale() || null == request.getNumPratica() || StringUtils.isBlank(request.getSpidCode()) || StringUtils.isBlank(request.getCodiceFiscale()) || StringUtils.isBlank(request.getNumPratica())) {
                response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
                log.info("END WITH BAD REQUEST");
                return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
            } else {
                boolean risultato = service.sendPushGestorePratiche(request);
                if (!risultato) {
                    response.setData("Errore nell'invio delle notifiche push");
                    response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
                    log.info("END WITH EXPECTION FAIL");
                    return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
                }
                response.setData("Notifiche push inviate con successo!");
                response.success(HttpStatus.SC_OK);
                log.info("END");
            }
        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        return ResponseEntity.ok(response);
    }

    @GetMapping(value = "/1.0/notificaPortaleNews", produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<String>> notificaportalenews(@RequestParam String idNews) {
        RisultatoDTO<String> response = new RisultatoDTO<String>();
        try {

            log.info("START");
            if (null == idNews || StringUtils.isBlank(idNews)) {
                response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
                log.info("END WITH BAD REQUEST");
                return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
            } else {
                boolean risultato = service.sendPushNews(idNews);
                if (!risultato) {
                    response.setData("Errore nell'invio delle notifiche push");
                    response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
                    log.info("END WITH EXPECTION FAIL");
                    return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
                }
                response.setData("Notifiche push inviate con successo!");
                response.success(HttpStatus.SC_OK);
                log.info("END");
            }
        } catch (Exception e) {
            response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            log.error("END WITH SERVER ERROR: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
        return ResponseEntity.ok(response);
    }

    @GetMapping(value = "/1.0/getProperty", produces = {MediaType.APPLICATION_JSON_VALUE})
    @CrossOrigin
    public ResponseEntity<RisultatoDTO<String>> getProperty(
            @RequestParam("property") String property) {
        log.info("START");
        log.info("Request: " + new Gson().toJson(property));

        RisultatoDTO<String> response = new RisultatoDTO<String>();
        String propValue;
        String messageLanguage = null;
        try {
            if (StringUtils.isBlank(property)) {
                log.error("End with bad request: " + property);
				response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
            }
            propValue = service.getProperty(property);

            if (StringUtils.isBlank(propValue)) {
                log.error("Proprieta' {} non trovata", property);
                messageLanguage = "Proprietà non presente";
                response.error(messageLanguage, HttpStatus.SC_NOT_FOUND);
                return ResponseEntity.status(HttpStatus.SC_NOT_FOUND).body(response);
            }
            response.setData(propValue);
            messageLanguage = "Proprietà recuperata con successo";
            response.setCode(HttpStatus.SC_OK);
            response.setDescrizione(messageLanguage);
			response.success(HttpStatus.SC_OK);
            log.info("Proprietà recuperata con successo" + new Gson().toJson(response.getData()));
            log.info("END");
            return ResponseEntity.ok(response);
        } catch (Exception e) {
            log.error("End with server error" + e.getMessage());
            response.error("Errore generale", HttpStatus.SC_INTERNAL_SERVER_ERROR);
            return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
        }
    }

}
