package it.itsvil.macs.rest;


import it.itsvil.macs.backend.dto.RisultatoDTO;
import it.itsvil.macs.backend.dto.ZoomMeetingObjectDTO;
import it.itsvil.macs.backend.dto.ZoomMeetingRequestDTO;
import it.itsvil.macs.backend.service.ZoomService;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@Log4j2
@RestController
@RequestMapping(value= "/zoom")
public class ZoomServiceRest {
	
	@Autowired
	private ZoomService service;
	

	@PostMapping(value = "/1.0/createMeeting" , consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<ZoomMeetingObjectDTO>> createMeeting(@RequestBody ZoomMeetingRequestDTO request){
		RisultatoDTO<ZoomMeetingObjectDTO> response = new RisultatoDTO<ZoomMeetingObjectDTO>();
		try {
			log.info("START");
			 {
				ZoomMeetingObjectDTO risultato=service.createMeeting(request);
				if (null == risultato) {
					response.setData(risultato);
					response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
					log.info("END WITH EXPECTION FAIL");
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
				response.setData(risultato);
				response.success(HttpStatus.SC_OK);
				log.info("END");				
			}
		}catch (Exception e) {
			e.printStackTrace();
			response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
			log.error("END WITH SERVER ERROR: {}"+e.getMessage());
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}


	@DeleteMapping(value = "/1.0/deleteMeeting" , produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> deleteMeeting(@RequestParam String id_meeting){
		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {
			log.info("START");
			{
				String risultato = service.deleteMeeting(id_meeting);

				response.setData(risultato);
				response.success(HttpStatus.SC_OK);
				log.info("END");
			}
		}catch (Exception e) {
			e.printStackTrace();
			response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
			log.error("END WITH SERVER ERROR: {}"+e.getMessage());
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}



}
