package it.itsvil.notificatore.rest;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import it.itsvil.notificatore.backend.dto.BroadcastPushNotificationRequestDTO;
import it.itsvil.notificatore.backend.dto.EmailDTO;
import it.itsvil.notificatore.backend.dto.PushNotificationDTO;
import it.itsvil.notificatore.backend.dto.RisultatoDTO;
import it.itsvil.notificatore.backend.service.NotificatoreEmailService;
import it.itsvil.notificatore.backend.service.NotificatorePushNotificationService;
import lombok.extern.log4j.Log4j2;

@Log4j2
@RestController
@RequestMapping(value= "/service")
public class NotificatoreServiceRest {

	@Autowired
	NotificatoreEmailService emailService;

	@Autowired
	NotificatorePushNotificationService pushService;

	@PostMapping(value = "/1.0/sendEmail", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> sendEmail10(@RequestHeader("ApplicationId") String applicationId, @RequestBody EmailDTO request){

		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {
			if (null != request && StringUtils.isNotBlank(request.getTextOgetto()) && StringUtils.isNotBlank(request.getTypeBody())) {
				List<String> destinatari = request.getDestinatari();
				if(null != destinatari && destinatari.size() > 0){
					boolean esito = emailService.sendEmail(applicationId, request);
					if(!esito){
						response.error("Impossibile effettuare l'invio",HttpStatus.SC_EXPECTATION_FAILED);
						return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
					}else{
						response.success(HttpStatus.SC_OK);
						response.setDescrizione("Email correttamente inviata");
					}
				}else {
					response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
					return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
	@PostMapping(value = "/1.0/sendBroadcastPushNotification", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> sendBroadcastPushNotification(@RequestHeader("ApplicationId") String applicationId, @RequestBody BroadcastPushNotificationRequestDTO push){

		RisultatoDTO<String> response = new RisultatoDTO<String>();

		try {
			if (null != push && null != push.getListOfToken() && push.getListOfToken().size() > 0 && StringUtils.isNotBlank(push.getPushTitle()) && StringUtils.isNotBlank(push.getPushMessage())) {
				boolean esito=false;
				esito = pushService.sendBroadcastPushNotification(applicationId, push);
				if(!esito){
					response.error("Impossibile effettuare l'invio",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}else{
					response.success(HttpStatus.SC_OK);
					response.setDescrizione("Notifica correttamente inviata");
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch(Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);

		}
		return ResponseEntity.ok(response);	
	}
	
	@PostMapping(value = "/1.0/sendPushNotification", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> sendPushNotification(@RequestHeader("ApplicationId") String applicationId, @RequestBody PushNotificationDTO push){

		RisultatoDTO<String> response = new RisultatoDTO<String>();

		try {
			if (null != push && null != push.getNotifications() && push.getNotifications().size() > 0) {
				boolean esito=false;
				if(!pushService.checkSendPushNotificationInput(push)){
					response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
					return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
				}
				esito = pushService.sendPushNotification(applicationId, push);
				if(!esito){
					response.error("Impossibile effettuare l'invio",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}else{
					response.success(HttpStatus.SC_OK);
					response.setDescrizione("Notifica correttamente inviata");
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch(Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);

		}
		return ResponseEntity.ok(response);	
	}

}
