package it.itsvil.profilatore.backend.service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;

import com.google.gson.Gson;

import it.itsvil.profilatore.backend.dao.ProfilatoreConfigurationDao;
import it.itsvil.profilatore.backend.dao.ProfilatoreDao;
import it.itsvil.profilatore.backend.dto.DettaglioFunzionalitaAggiuntiveDTO;
import it.itsvil.profilatore.backend.dto.DettaglioFunzioneDTO;
import it.itsvil.profilatore.backend.dto.DettaglioRuoloDTO;
import it.itsvil.profilatore.backend.dto.DettaglioUtenteDTO;
import it.itsvil.profilatore.backend.dto.FunzionalitaDTO;
import it.itsvil.profilatore.backend.dto.FunzioneDTO;
import it.itsvil.profilatore.backend.dto.FunzioneFunzionalitaDTO;
import it.itsvil.profilatore.backend.dto.InsertDettaglioFunzioneRequestDTO;
import it.itsvil.profilatore.backend.dto.InsertDettaglioRuoloRequestDTO;
import it.itsvil.profilatore.backend.dto.InsertFunzionalitaRequestDTO;
import it.itsvil.profilatore.backend.dto.InsertUtenzaByRuoloRequestDTO;
import it.itsvil.profilatore.backend.dto.InsertUtenzaRequestDTO;
import it.itsvil.profilatore.backend.dto.RuoloDTO;
import it.itsvil.profilatore.backend.dto.UpdateUtenzaRequestDTO;
import it.itsvil.profilatore.backend.dto.UtenzeDTO;
import it.itsvil.profilatore.backend.utils.ProfilatoreEnum;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service (value = "ProfilatoreServiceImpl")
public class ProfilatoreServiceImpl implements ProfilatoreService{

	@Autowired
	ProfilatoreDao dao;	
	
	@Autowired
	ProfilatoreConfigurationDao configurationDao;	
	
	@Autowired
	private PlatformTransactionManager transactionManager;

	@Override
	public DettaglioRuoloDTO getDettaglioRuolo(String t_nome_ruolo, String n_id_app) {
		DettaglioRuoloDTO risultato = new DettaglioRuoloDTO();
		List<DettaglioFunzioneDTO> funzioni=new ArrayList<DettaglioFunzioneDTO>();
		try {

			RuoloDTO ruolo=dao.getRuolo(null,t_nome_ruolo, n_id_app);
			risultato.setT_nome(ruolo.getT_nome());
			risultato.setT_descrizione(ruolo.getT_descrizione());
			risultato.setN_id_ruolo(ruolo.getN_id_ruolo());
			risultato.setD_data_ins(ruolo.getD_data_ins());

			List<FunzioneDTO> lista_funz= dao.getFunzione(risultato.getN_id_ruolo());
			for(FunzioneDTO funz:lista_funz) {
				DettaglioFunzioneDTO funzione=new DettaglioFunzioneDTO();
				funzione.setT_nome(funz.getT_nome());
				funzione.setT_descrizione(funz.getT_descrizione());
				funzione.setN_id_funzione(funz.getN_id_funzione());
				funzione.setD_data_ins(funz.getD_data_ins());
				funzioni.add(funzione);	
				List<FunzionalitaDTO> all_funzionalita=dao.getFunzionalitaAttive(funzione.getN_id_funzione());
				funzione.setFunzionalita(all_funzionalita);
			}
			risultato.setFunzioni(funzioni);
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}

		return risultato;
	}
	
	@Override
	public DettaglioRuoloDTO getDettaglioRuoloById(Long n_id_ruolo, String n_id_app) {
		DettaglioRuoloDTO risultato = new DettaglioRuoloDTO();
		List<DettaglioFunzioneDTO> funzioni=new ArrayList<DettaglioFunzioneDTO>();
		try {

			RuoloDTO ruolo=dao.getRuolo(n_id_ruolo,null, n_id_app);
			risultato.setT_nome(ruolo.getT_nome());
			risultato.setT_descrizione(ruolo.getT_descrizione());
			risultato.setN_id_ruolo(ruolo.getN_id_ruolo());
			risultato.setD_data_ins(ruolo.getD_data_ins());

			List<FunzioneDTO> lista_funz= dao.getFunzione(risultato.getN_id_ruolo());
			for(FunzioneDTO funz:lista_funz) {
				DettaglioFunzioneDTO funzione=new DettaglioFunzioneDTO();
				funzione.setT_nome(funz.getT_nome());
				funzione.setT_descrizione(funz.getT_descrizione());
				funzione.setN_id_funzione(funz.getN_id_funzione());
				funzione.setD_data_ins(funz.getD_data_ins());
				funzioni.add(funzione);	
				List<FunzionalitaDTO> all_funzionalita=dao.getFunzionalitaAttive(funzione.getN_id_funzione());
				funzione.setFunzionalita(all_funzionalita);
			}
			risultato.setFunzioni(funzioni);
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}

		return risultato;
	}

	@Override
	public int updateUtenza(UpdateUtenzaRequestDTO request, String applicationId) {
		TransactionStatus txStatus = transactionManager.getTransaction(new DefaultTransactionDefinition());
		try {
			UtenzeDTO utenza = new UtenzeDTO();
			utenza.setN_id_app(applicationId);
			utenza.setB_attivo(request.getB_attivo());
			utenza.setN_id_ruolo(request.getN_id_ruolo());
			utenza.setT_id_user(request.getT_id_user());
			dao.updateUtenza(utenza);
			transactionManager.commit(txStatus);
			return 0;
		}catch(Exception e){
			log.error("ERRORE: {}", e.getMessage());
			transactionManager.rollback(txStatus);
			return -1;
		}
	}
	
	@Override
	public int insertDettaglioRuolo(InsertDettaglioRuoloRequestDTO request, String n_id_app) {
		
		List<InsertDettaglioFunzioneRequestDTO> funzioni=new ArrayList<InsertDettaglioFunzioneRequestDTO>();
		FunzioneFunzionalitaDTO f2 = new FunzioneFunzionalitaDTO();
		funzioni = request.getFunzioni();
		
		TransactionStatus txStatus = transactionManager.getTransaction(new DefaultTransactionDefinition());
		RuoloDTO ruolo = new RuoloDTO();
		
		ruolo = dao.getRuolo(null, request.getT_nome_ruolo(), n_id_app);
		
		if ( null == ruolo) {
			ruolo = new RuoloDTO();
			try {
				Long n_id_ruolo = configurationDao.getSequenceRuolo();
				ruolo.setN_id_ruolo(n_id_ruolo);
				ruolo.setN_id_app(n_id_app);
				ruolo.setD_data_ins(LocalDateTime.now());
				ruolo.setB_attivo(ProfilatoreEnum.ATTIVO.getCodice());
				ruolo.setT_descrizione(request.getT_descrizione_ruolo());
				ruolo.setT_nome(request.getT_nome_ruolo());
				configurationDao.insertRuolo(ruolo);
				
	
				for(InsertDettaglioFunzioneRequestDTO funz:funzioni) {
					FunzioneDTO funzione=new FunzioneDTO();
					funzione.setN_id_ruolo(n_id_ruolo);
					funzione.setT_nome(funz.getT_nome_funzione());
					funzione.setT_descrizione(funz.getT_descrizione_funzione());
					Long n_id_funzione = configurationDao.nextFunzioneSequence();
					funzione.setN_id_funzione(n_id_funzione);
					funzione.setB_attivo(ProfilatoreEnum.ATTIVO.getCodice());
					funzione.setD_data_ins(LocalDateTime.now());
					configurationDao.insertFunzione(funzione);
					List<InsertFunzionalitaRequestDTO> all_funzionalita=funz.getFunzionalita();
					for(InsertFunzionalitaRequestDTO ff:all_funzionalita) {
						Long n_id_funzionalita = configurationDao.getSqFunzionalita();
						FunzionalitaDTO funzionalita = new FunzionalitaDTO();
						funzionalita.setN_id_funzionalita(n_id_funzionalita);
						funzionalita.setD_data(LocalDateTime.now());
						funzionalita.setB_attivo(ProfilatoreEnum.ATTIVO.getCodice());
						funzionalita.setT_descrizione(ff.getT_descrizione_funzionalita());
						funzionalita.setT_livello_sicurezza(ff.getT_livello_sicurezza());
						funzionalita.setT_nome(ff.getT_nome_funzionalita());
						funzionalita.setT_info_aggiuntive(ff.getT_info_aggiuntive());
					    configurationDao.insertFunzionalita(funzionalita);
						f2.setB_attivo(ProfilatoreEnum.ATTIVO.getCodice());
						f2.setD_data_ins(LocalDateTime.now());
						f2.setN_id_funzione(n_id_funzione);
						f2.setN_id_funzionalita(n_id_funzionalita);
						configurationDao.insertFunzioneFunzionalita(f2);
					}
				}
				
				transactionManager.commit(txStatus);
				return 0;
			
			}catch (Exception e) {
				log.error("ERRORE: {}", e.getMessage());
				transactionManager.rollback(txStatus);
				return -1;
			}
		}
		
		else return 1;
		
	}


	@Override
	public DettaglioUtenteDTO getDettaglioUtenze(String t_id_user, String applicationId) {
		DettaglioUtenteDTO risultato = new DettaglioUtenteDTO();

		try {
			UtenzeDTO utente = dao.getUtente(t_id_user, applicationId);
			if(null != utente) {
				risultato.setT_id_user(utente.getT_id_user());
				risultato.setN_id_ruolo(utente.getN_id_ruolo());
				risultato.setD_data_ins(utente.getD_data_ins());
				DettaglioRuoloDTO roole=new DettaglioRuoloDTO();
				Long id_ruolo = utente.getN_id_ruolo();
				roole=getDettaglioRuoloById(id_ruolo, applicationId);
				Long n_id_utente=utente.getN_id_utente();
				List<DettaglioFunzionalitaAggiuntiveDTO> funzionalita_aggi=dao.getFunzionalitaAggiuntiveAttive(n_id_utente);
				if(null!=funzionalita_aggi && funzionalita_aggi.size()>0) {
					int position=-1;
					for(DettaglioFunzionalitaAggiuntiveDTO agg:funzionalita_aggi) {
						position=checkFunzione(agg, roole);
						if(position==-1){
							roole=aggiungiFunzioneFunzionalita(agg,roole);

						}else {
							roole=aggiungiFunzionalita(position,agg,roole);
						}

						position=-1;
					}
				}

				risultato.setRuolo(roole);
			}else {
				return null;
			}
		}catch(Exception e){
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}

		return risultato;
	}



	private int checkFunzione(DettaglioFunzionalitaAggiuntiveDTO f, DettaglioRuoloDTO d) {
		int p=0;

		Long id_funzione_aggiuntiva =f.getN_id_funzione();
		List<DettaglioFunzioneDTO> lista_funzioni=d.getFunzioni();
		for(DettaglioFunzioneDTO funz:lista_funzioni) {
			Long n_id_fuzione=funz.getN_id_funzione();
			if(n_id_fuzione.equals(id_funzione_aggiuntiva)) {
				p=lista_funzioni.indexOf(funz);
				return p;
			}else {
				p=-1;
			}
		}
		return p;
	}



	private DettaglioRuoloDTO aggiungiFunzioneFunzionalita(DettaglioFunzionalitaAggiuntiveDTO f, DettaglioRuoloDTO d) {

		DettaglioFunzioneDTO nuova_funzione=new DettaglioFunzioneDTO();
		nuova_funzione.setN_id_funzione(f.getN_id_funzione());
		nuova_funzione.setT_nome(f.getT_nome_funzione());
		nuova_funzione.setT_descrizione(f.getT_descrizione_funzione());
		nuova_funzione.setB_attivo(f.getB_attivo_funzione());
		nuova_funzione.setD_data_ins(f.getD_data_funzione());
		List<FunzionalitaDTO> lista_funzionalita= new ArrayList<FunzionalitaDTO>();
		nuova_funzione.setFunzionalita(lista_funzionalita);

		FunzionalitaDTO nuova_funzionalita=new FunzionalitaDTO();
		nuova_funzionalita.setN_id_funzionalita(f.getN_id_funzionalita());
		nuova_funzionalita.setT_nome(f.getT_nome_funzionalita());
		nuova_funzionalita.setT_descrizione(f.getT_descrizione_funzionalita());
		nuova_funzionalita.setD_data(f.getD_data_funzionalita());
		nuova_funzionalita.setB_attivo(f.getB_attivo_funzionalita());
		nuova_funzionalita.setT_livello_sicurezza(f.getT_livello_sicurezza());

		nuova_funzione.getFunzionalita().add(nuova_funzionalita);

		d.getFunzioni().add(nuova_funzione);
		return d;
	}

	private DettaglioRuoloDTO aggiungiFunzionalita(int position, DettaglioFunzionalitaAggiuntiveDTO f, DettaglioRuoloDTO d) {
		boolean trovato = false;
		List<DettaglioFunzioneDTO> lista_funzioni=d.getFunzioni();
		DettaglioFunzioneDTO funzione_presente=lista_funzioni.get(position);
		List<FunzionalitaDTO> funzionalita_presenti= funzione_presente.getFunzionalita();
		Long id_funzionalita_agg=f.getN_id_funzionalita();


		for (FunzionalitaDTO funzionalitaDTO : funzionalita_presenti) {
			if(funzionalitaDTO.getN_id_funzionalita().equals(id_funzionalita_agg)) {
				trovato = true;
			}
		}

		if(!trovato) {
			FunzionalitaDTO nuova=new FunzionalitaDTO();
			nuova.setN_id_funzionalita(f.getN_id_funzionalita());
			nuova.setT_nome(f.getT_nome_funzionalita());
			nuova.setT_descrizione(f.getT_descrizione_funzionalita());
			nuova.setB_attivo(f.getB_attivo());
			nuova.setD_data(f.getD_data_funzionalita());
			nuova.setT_livello_sicurezza(f.getT_livello_sicurezza());
			d.getFunzioni().get(position).getFunzionalita().add(nuova);
		}

		return d;
	}
	
	@Override
	public int insertUtenza(InsertUtenzaRequestDTO request, String n_id_app) {
		TransactionStatus txStatus = transactionManager.getTransaction(new DefaultTransactionDefinition());
		try {
			UtenzeDTO utente = new UtenzeDTO();
			LocalDateTime now =  LocalDateTime.now();
			utente.setD_data_ins(now);
			utente.setB_attivo(ProfilatoreEnum.ATTIVO.getCodice());
			Long n_id_utente = dao.nextUserSequence();
			utente.setN_id_utente(n_id_utente);
			utente.setN_id_app(n_id_app);
			utente.setN_id_ruolo(request.getN_id_ruolo());
			utente.setT_id_user(request.getT_id_user());
			dao.insertUtenza(utente);
			transactionManager.commit(txStatus);
			return 0;
		}catch(Exception e){
			log.error("ERRORE: {}", e.getMessage());
			transactionManager.rollback(txStatus);
			return -1;
		}
	}
	
	@Override
	public int insertUtenzaByRuolo(InsertUtenzaByRuoloRequestDTO request, String n_id_app) {
		log.info("***START***");
		
		TransactionStatus txStatus = transactionManager.getTransaction(new DefaultTransactionDefinition());
		
		try {
			//Recupero dettaglio ruolo
			DettaglioRuoloDTO ruolo = getDettaglioRuolo(request.getT_nome_ruolo(), n_id_app);
			if(null == ruolo) {
				log.error("ERRORE: Errore durante il recupero del dettaglio del ruolo");
				return -1;
			}
			
			//Verifica presenza utente
			UtenzeDTO utenza = dao.getUtente(request.getT_id_user(), n_id_app);
			if(null != utenza) {
				log.info("Utenza gia' presente");
				return 1;
			}
			
			//Creazione utenza da inserire
			var utenteToInsert = new UtenzeDTO();
			
			utenteToInsert.setN_id_utente(dao.nextUserSequence());
			utenteToInsert.setN_id_app(n_id_app);
			utenteToInsert.setN_id_ruolo(ruolo.getN_id_ruolo());
			utenteToInsert.setT_id_user(request.getT_id_user());
			utenteToInsert.setB_attivo(ProfilatoreEnum.ATTIVO.getCodice());
			utenteToInsert.setD_data_ins(LocalDateTime.now());
			
			log.info("UTENTE DA INSERIRE: {}", new Gson().toJson(utenteToInsert));
		
			dao.insertUtenza(utenteToInsert);
			
			transactionManager.commit(txStatus);
			
			log.info("***END***");
			return 0;
		}catch(Exception e){
			log.error("ERRORE: {}", e.getMessage());
			transactionManager.rollback(txStatus);
			return -1;
		}
	}


}
